"""network unittest"""
import unittest
import network as pynet
import random
random.seed(10)
import pysal

class Network_Tester(unittest.TestCase):

    def setUp(self):
        self.net = 'streets.shp'
        self.G = pynet.read_network(self.net)
        self.G2 = {(1,1): {(2,2): 0.125, (3,3): 0.75}, (2,2): {(1,1): 0.125, (4,4): 1.2},
                   (3,3): {(1,1): 0.75, (4,4): 0.375},
                   (4,4): {(2,2): 1.2, (3,3): 0.375, (5,5): 0.5},
                   (5,5): {(4,4): 0.5}, (6,6):{(7,7):1.0}, (7,7):{(6,6):1.0}}
        self.GDirected = {(1,1): {(2,2): 0.125, (3,3): 0.75}, (2,2): {(1,1): 0.125},
                   (3,3): {(1,1): 0.75, (4,4): 0.375},
                   (4,4): {(2,2): 1.2, (3,3): 0.375, (5,5): 0.5},
                   (5,5): {(4,4): 0.5}, (6,6):{(7,7):1.0}, (7,7):{(6,6):1.0}}
        self.points = [((4,4), (2,2), 0.51466686561013752, 0.68533313438986243), 
                       ((4,4), (3,3), 0.077286837052313151, 0.29771316294768685), 
                       ((6,6), (7,7), 0.82358887253344548, 0.17641112746655452)]

    def test_no_nodes(self):
        self.assertEqual(pynet.no_nodes(self.G), 230)

    def test_no_edges(self):
        self.assertEqual(pynet.no_edges(self.G), 303)

    def test_tot_net_length(self):
        self.assertAlmostEqual(pynet.tot_net_length(self.G), 52207.04600797734, places=1)

    def test_walk(self):
        correct_path = {(5, 5): {(4, 4): 0.5}, (2, 2): {(4, 4): 1.2, (1, 1): 0.125}, 
                        (1, 1): {(3, 3): 0.75, (2, 2): 0.125}, 
                        (4, 4): {(5, 5): 0.5, (3, 3): 0.375, (2, 2): 1.2}, 
                        (3, 3): {(4, 4): 0.375, (1, 1): 0.75}}
        traversal_path = pynet.walk(self.G2, (1,1))
        self.assertEqual(traversal_path, correct_path)        

    def test_components(self):
        components = pynet.components(self.G)
        self.assertEqual(pynet.no_nodes(components[0]), 230)        
        components = pynet.components(self.G2)
        self.assertEqual(len(components), 2)        

    def test_no_components(self):
        no_components = pynet.no_components(self.G2)
        self.assertEqual(no_components, 2)        

    def test_net_global_stats(self):
        stats = ['v', 'e', 'L', 'p', 'u', 'alpha', 'beta', 'emax', 'gamma', 'eta', 
                 'net_den', 'detour']
        values = pynet.net_global_stats(self.G, detour=True)
        values = dict(zip(stats, values))
        self.assertEqual(values['v'], 230)
        self.assertEqual(values['e'], 303)
        self.assertAlmostEqual(values['L'], 52207.04600797734, places=1)
        self.assertAlmostEqual(values['eta'], 172.30048187451268)
        self.assertAlmostEqual(values['beta'], 1.317391304347826, places=2)
        self.assertAlmostEqual(values['emax'], 684)
        self.assertAlmostEqual(values['gamma'], 0.44298245614035087, places=2)
        self.assertAlmostEqual(values['detour'], 0.78002937059822186, places=4)
        for k in values:
            print k, values[k]

    def test_random_projs(self):
        random.seed(10)
        random_points = pynet.random_projs(self.G2, 3)
        points = [((7, 7), (6, 6), 0.42888905467511462, 0.57111094532488538), 
                  ((7, 7), (6, 6), 0.20609823213950174, 0.79390176786049826), 
                  ((1, 1), (3, 3), 0.61769165440008411, 0.13230834559991589)] 
        self.assertEqual(random_points, points)
        
    def test_proj_distances_undirected(self):
        source = self.points[0]
        destinations = self.points[1:]
        distances = pynet.proj_distances_undirected(self.G2, source, destinations, r=1.0)
        self.assertAlmostEqual(distances.values()[0], 0.59195370266245062)

    def test_proj_distances_directed(self):
        source = self.points[0]
        destinations = self.points
        distances = pynet.proj_distances_directed(self.G2, source, destinations)
        distance_values = [0.0, 1.9626199714421757]
        self.assertEqual(distances.values(), distance_values)

    def test_dijkstras(self):
        distances = pynet.dijkstras(self.G2, (1,1))
        distance_values = {(5, 5): 1.625, (2, 2): 0.125, (1, 1): 0, (4, 4): 1.125, (3, 3): 0.75}
        self.assertEqual(distances, distance_values)

    def test_snap(self):
        snapper = pynet.Snapper(self.G2)
        projected_point = snapper.snap((2.5,2.5))
        self.assertEqual(projected_point, ((2, 2),(4, 4),3.9252311467094367e-17,1.1775693440128314e-16))

    def test_network_from_endnodes(self):
        shape = pysal.open(self.net)
        dbf = pysal.open(self.net[:-3] + 'dbf')
        def weight(geo_object, record):
            return 1
        graph = pynet.network_from_endnodes(shape, dbf, weight)
        neighbors = {(724432.38723173144, 877800.08747069736): 1, 
                     (725247.70571468933, 877812.36851842562): 1}
        start_point = (724587.78057580709, 877802.4281426128)
        self.assertEqual(graph[start_point], neighbors)

    def test_network_from_allvertices(self):
        shape = pysal.open(self.net)
        dbf = pysal.open(self.net[:-3] + 'dbf')
        graph = pynet.network_from_allvertices(shape, dbf)
        neighbors = {(724432.38723173144, 877800.08747069736): 155.41097171058956, 
                     (725247.70571468933, 877812.36851842562): 660.00000000003809}
        start_point = (724587.78057580709, 877802.4281426128)
        self.assertEqual(graph[start_point], neighbors)

    def test_read_hierarchical_network(self):
        shape = pysal.open(self.net)
        dbf = pysal.open(self.net[:-3] + 'dbf')
        graph_detail, graph_endnode, link = pynet.read_hierarchical_network(shape, dbf)
        neighbors = {(725220.77363919443, 880985.09708712087): 659.99999999994725, 
                     (724400.64597190416, 880984.45593258401): 160.12791790928244}
        vertex1 = (724560.77384088072, 880984.58111639845)
        vertex2 = (724400.64597190416, 880984.45593258401)
        self.assertEqual(graph_detail[vertex1], neighbors)
        self.assertEqual(graph_endnode[vertex1], neighbors)
        self.assertEqual(link[(vertex1, vertex2)][0], (vertex1, vertex2))

    def test_read_network(self):
        graph = pynet.read_network(self.net)
        self.assertEqual(graph, self.G) 

    def test_proj_pnt_coor(self):
        point = pynet.proj_pnt_coor(self.points[0])
        point_value = (3.6360755692750462, 3.6360755692750462)
        self.assertEqual(point, point_value)

    def test_inject_points(self):
        graph, point_coors = pynet.inject_points(self.G2, self.points)
        self.assertEqual(len(graph), len(self.G2) + 3)

    def test_mesh_network(self):
        graph = pynet.mesh_network(self.G2, 0.1)
        self.assertEqual(len(graph), 41)

    def test_write_network_to_pysalshp(self):
        out = 'output_network.shp'
        pynet.write_network_to_pysalshp(self.G, out)
        graph = pynet.read_network(out)
        self.assertEqual(graph, self.G)    

    def test_write_valued_network_to_shp(self):
        out = 'output_network.shp'
        fields = ['WEIGHT1','WEIGHT2']
        types = [('N',7,3),('N',7,3)]
        values = {(1,1):1,(2,2):2,(3,3):3,(4,4):4,(5,5):5,(6,6):6,(7,7):7}
        def doubleX(values, node):
            return values[node]*2.0
        pynet.write_valued_network_to_shp(out,fields,types,self.G2,values,doubleX)
        graph = pynet.read_network(out)
        self.assertEqual(len(graph), len(self.G2))    

    def test_list_network_to_shp(self):
        out = 'output_network.shp'
        fields = ['WEIGHT']
        types = [('N',7,3)]
        list_network = []
        for node1 in self.G2:
            for node2 in self.G2[node1]:
                list_network.append(((node1, node2),random.random()))
        pynet.write_list_network_to_shp(out,fields,types,list_network)
        graph = pynet.read_network(out)
        self.assertEqual(len(graph), len(self.G2))    


suite = unittest.TestSuite()
test_classes = [Network_Tester]
for i in test_classes:
    a = unittest.TestLoader().loadTestsFromTestCase(i)
    suite.addTest(a)

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite)



import pysal
import types as ty
import sys

PY3 = sys.version_info.major > 2
from pysal.common import iteritems as diter

if not PY3:
    clstypes = (type, ty.ClassType)
else:
    clstypes = type

def _find_bcs():
    classes = dict()
    bcs = dict()
    ucs = dict()
    submods = dict()
    for obname, ob in diter(pysal.spreg.__dict__):
        if isinstance(ob, ty.ModuleType):
            if ob.__package__.startswith("pysal"):
                submods.update({obname:ob})
        elif isinstance(ob, clstypes):
            classes.update({obname:ob})
            if ob.__name__.startswith('Base'):
                bcs.update({obname:ob})
    for modname, mod in diter(submods):
        basecands = dict()
        for clname in dir(mod):
            cl = mod.__dict__[clname]
            if isinstance(cl, clstypes):
                try:
                    if cl.__name__.startswith('Base'):
                        if cl not in bcs:
                            bcs.update({cl.__name__:cl})
                        else:
                            classes.update({cl.__name__:cl})
                except:
                    pass
    ucs.update({k:v for k,v in diter(classes) if (
                any([issubclass(v, bc) for bc in bcs.values()])
                and (k not in bcs))
                or k.endswith('Regimes')})
    return bcs, ucs

class Namespace:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

base, user = _find_bcs()
base = Namespace(**base)
user = Namespace(**user)
everything = Namespace(**base.__dict__)
everything.__dict__.update(user.__dict__)

#if we go with something like a "base" and "user" submodule setup,
#it'd be as simple as flattening the subclasses out into those submodules. 
#for name, cl in diter(_find_bcs()[0]):
#    exec('{n} = {cls}'.format(n=name, cls=cl))



import unittest
import pysal
from pysal.core.IOHandlers.geobugs_txt import GeoBUGSTextIO
import tempfile
import os


class test_GeoBUGSTextIO(unittest.TestCase):
    def setUp(self):
        self.test_file_scot = test_file_scot = pysal.examples.get_path(
            'geobugs_scot')
        self.test_file_col = test_file_col = pysal.examples.get_path(
            'spdep_listw2WB_columbus')
        self.obj_scot = GeoBUGSTextIO(test_file_scot, 'r')
        self.obj_col = GeoBUGSTextIO(test_file_col, 'r')

    def test_close(self):
        for obj in [self.obj_scot, self.obj_col]:
            f = obj
            f.close()
            self.failUnlessRaises(ValueError, f.read)

    def test_read(self):
        w_scot = self.obj_scot.read()
        self.assertEqual(56, w_scot.n)
        self.assertEqual(4.1785714285714288, w_scot.mean_neighbors)
        self.assertEqual([1.0, 1.0, 1.0], w_scot[1].values())

        w_col = self.obj_col.read()
        self.assertEqual(49, w_col.n)
        self.assertEqual(4.6938775510204085, w_col.mean_neighbors)
        self.assertEqual([0.5, 0.5], w_col[1].values())

    def test_seek(self):
        self.test_read()
        self.failUnlessRaises(StopIteration, self.obj_scot.read)
        self.failUnlessRaises(StopIteration, self.obj_col.read)
        self.obj_scot.seek(0)
        self.obj_col.seek(0)
        self.test_read()

    def test_write(self):
        for obj in [self.obj_scot, self.obj_col]:
            w = obj.read()
            f = tempfile.NamedTemporaryFile(
                suffix='', dir=pysal.examples.get_path(''))
            fname = f.name
            f.close()
            o = pysal.open(fname, 'w', 'geobugs_text')
            o.write(w)
            o.close()
            wnew = pysal.open(fname, 'r', 'geobugs_text').read()
            self.assertEqual(wnew.pct_nonzero, w.pct_nonzero)
            os.remove(fname)

if __name__ == '__main__':
    unittest.main()



from warnings import warn as Warn
from ..cg import asShape
from ..core.FileIO import FileIO
from .weights import W, WSP
from ._contW_binning import ContiguityWeightsPolygons
from ._contW_rtree import ContiguityWeights_rtree
from ._contW_lists import ContiguityWeightsLists
from .util import get_ids
WT_TYPE = {'rook': 2, 'queen': 1}  # for _contW_Binning

__author__ = "Sergio J. Rey <srey@asu.edu> , Levi John Wolf <levi.john.wolf@gmail.com>"

class Rook(W):
    def __init__(self, polygons, method='binning', **kw):
        """
        Construct a weights object from a collection of pysal polygons.

        Arguments
        ---------
        polygons    : list
                      a collection of PySAL shapes to build weights from
        ids         : list
                      a list of names to use to build the weights
        method      : string
                      the algorithm to use to construct the method
        **kw        : keyword arguments
                      optional arguments for :class:`pysal.weights.W`

        See Also
        ---------
        :class:`pysal.weights.W`
        """
        criterion = 'rook'
        ids = kw.pop('ids', None)
        neighbors, ids = _build(polygons, ids=ids, 
                                criterion=criterion, method=method)
        W.__init__(self, neighbors, ids=ids, **kw)
    
    @classmethod
    def from_shapefile(cls, filepath, idVariable=None, full=False, **kwargs):
        """
        Rook contiguity weights from a polygon shapefile.

        Parameters
        ----------

        shapefile : string
                    name of polygon shapefile including suffix.
        sparse    : boolean
                    If True return WSP instance
                    If False return W instance

        Returns
        -------

        w          : W
                     instance of spatial weights

        Examples
        --------
        >>> wr=rook_from_shapefile(pysal.examples.get_path("columbus.shp"), "POLYID")
        >>> "%.3f"%wr.pct_nonzero
        '8.330'
        >>> wr=rook_from_shapefile(pysal.examples.get_path("columbus.shp"), sparse=True)
        >>> pct_sp = wr.sparse.nnz *1. / wr.n**2
        >>> "%.3f"%pct_sp
        '0.083'

        Notes
        -----

        Rook contiguity defines as neighbors any pair of polygons that share a
        common edge in their polygon definitions.

        See Also
        --------
        :class:`pysal.weights.W`
        :class:`pysal.weights.Rook`
        """
        sparse = kwargs.pop('sparse', False)
        if idVariable is not None:
            ids = get_ids(filepath, idVariable) 
        else:
            ids = None
        iterable = FileIO(filepath)
        w = cls(FileIO(filepath), ids=ids, **kwargs)
        w.set_shapefile(filepath, idVariable=idVariable, full=full)
        if sparse:
            w = w.to_WSP()
        return w
    
    @classmethod
    def from_iterable(cls, iterable, sparse=False, **kwargs):
        """
        Construct a weights object from a collection of arbitrary polygons. This
        will cast the polygons to PySAL polygons, then build the W.

        Arguments
        ---------
        iterable    : iterable
                      a collection of of shapes to be cast to PySAL shapes. Must
                      support iteration. Contents should at least implement a
                      `__geo_interface__` attribute or be able to be coerced to
                      geometries using pysal.cg.asShape
        **kw        : keyword arguments
                      optional arguments for  :class:`pysal.weights.W`
        See Also
        ----------
        :class:`pysal.weights.W`
        :class:`pysal.weights.Rook`
        """
        new_iterable = [asShape(shape) for shape in iterable]
        
        w = cls(new_iterable, **kwargs) 
        if sparse:
            w = WSP.from_W(w)
        
        return w
    
    @classmethod
    def from_dataframe(cls, df, geom_col='geometry', **kwargs):
        """
        Construct a weights object from a pandas dataframe with a geometry
        column. This will cast the polygons to PySAL polygons, then build the W
        using ids from the dataframe.

        Arguments
        ---------
        df          : DataFrame
                      a :class: `pandas.DataFrame` containing geometries to use
                      for spatial weights
        geom_col    : string
                      the name of the column in `df` that contains the
                      geometries. Defaults to `geometry`
        idVariable  : string
                      the name of the column to use as IDs. If nothing is
                      provided, the dataframe index is used
        ids         : list
                      a list of ids to use to index the spatial weights object.
                      Order is not respected from this list.
        id_order    : list
                      an ordered list of ids to use to index the spatial weights
                      object. If used, the resulting weights object will iterate
                      over results in the order of the names provided in this
                      argument. 

        See Also
        ---------
        :class:`pysal.weights.W`
        :class:`pysal.weights.Rook`
        """
        idVariable = kwargs.pop('idVariable', None)
        ids = kwargs.pop('ids', None)
        id_order = kwargs.pop('id_order', True)
        if id_order is True and ((idVariable is not None) 
                                  or (ids is not None)):
                # if idVariable is None, we want ids. Otherwise, we want the
                # idVariable column
            ids = list(df.get(idVariable, ids))
            id_order = ids
        elif isinstance(id_order, str):
            ids = df.get(id_order, ids)
            id_order = ids
        elif idVariable is not None:
            ids = df.get(idVariable).tolist()
        elif isinstance(ids, str):
            ids = df.get(ids).tolist()
        else:
            id_order = list(df.index)
            ids = list(df.index)
        w = cls.from_iterable(df[geom_col].tolist(), ids=ids, id_order=id_order, **kwargs)
        return w

class Queen(W):
    def __init__(self, polygons,method='binning', **kw):
        """
        Construct a weights object from a collection of pysal polygons.

        Arguments
        ---------
        polygons    : list
                      a collection of PySAL shapes to build weights from
        ids         : list
                      a list of names to use to build the weights
        method      : string
                      the algorithm to use to construct the method
        **kw        : keyword arguments
                      optional arguments for :class:`pysal.weights.W`

        See Also
        ---------
        :class:`pysal.weights.W`
        """
        criterion = 'queen'
        ids = kw.pop('ids', None)
        neighbors, ids = _build(polygons, ids=ids, 
                                criterion=criterion, method=method)
        W.__init__(self, neighbors, ids=ids, **kw)
    
    @classmethod
    def from_shapefile(cls, filepath, idVariable=None, full=False, **kwargs):
        """
        Queen contiguity weights from a polygon shapefile.

        Parameters
        ----------

        shapefile   : string
                      name of polygon shapefile including suffix.
        idVariable  : string
                      name of a column in the shapefile's DBF to use for ids.
        sparse      : boolean
                      If True return WSP instance
                      If False return W instance
        Returns
        -------

        w            : W
                       instance of spatial weights

        Examples
        --------
        >>> wq=Queen.from_shapefile(pysal.examples.get_path("columbus.shp"))
        >>> "%.3f"%wq.pct_nonzero
        '9.829'
        >>> wq=Queen.from_shapefile(pysal.examples.get_path("columbus.shp"),"POLYID")
        >>> "%.3f"%wq.pct_nonzero
        '9.829'
        >>> wq=Queen.from_shapefile(pysal.examples.get_path("columbus.shp"), sparse=True)
        >>> pct_sp = wq.sparse.nnz *1. / wq.n**2
        >>> "%.3f"%pct_sp
        '0.098'

        Notes

        Queen contiguity defines as neighbors any pair of polygons that share at
        least one vertex in their polygon definitions.

        See Also
        --------
        :class:`pysal.weights.W`
        :class:`pysal.weights.Queen`
        """
        sparse = kwargs.pop('sparse', False)
        if idVariable is not None:
            ids = get_ids(filepath, idVariable) 
        else:
            ids = None
        iterable = FileIO(filepath)
        w = cls(FileIO(filepath), ids=ids, **kwargs)
        w.set_shapefile(filepath, idVariable=idVariable, full=full)
        if sparse:
            w = w.to_WSP()
        return w

    @classmethod
    def from_iterable(cls, iterable, sparse=False, **kwargs):
        """
        Construct a weights object from a collection of arbitrary polygons. This
        will cast the polygons to PySAL polygons, then build the W.

        Arguments
        ---------
        iterable    : iterable
                      a collection of of shapes to be cast to PySAL shapes. Must
                      support iteration. Contents should at least implement a
                      `__geo_interface__` attribute or be able to be coerced to
                      geometries using pysal.cg.asShape
        **kw        : keyword arguments
                      optional arguments for  :class:`pysal.weights.W`
        See Also
        ----------
        :class:`pysal.weights.W`
        :class:`pysal.weights.Queen`
        """
        new_iterable = [asShape(shape) for shape in iterable]
        
        w = cls(new_iterable, **kwargs) 
        if sparse:
            w = WSP.from_W(w)
        
        return w

    @classmethod
    def from_dataframe(cls, df, geom_col='geometry', **kwargs):
        """
        Construct a weights object from a pandas dataframe with a geometry
        column. This will cast the polygons to PySAL polygons, then build the W
        using ids from the dataframe.

        Arguments
        ---------
        df          : DataFrame
                      a :class: `pandas.DataFrame` containing geometries to use
                      for spatial weights
        geom_col    : string
                      the name of the column in `df` that contains the
                      geometries. Defaults to `geometry`
        idVariable  : string
                      the name of the column to use as IDs. If nothing is
                      provided, the dataframe index is used
        ids         : list
                      a list of ids to use to index the spatial weights object.
                      Order is not respected from this list.
        id_order    : list
                      an ordered list of ids to use to index the spatial weights
                      object. If used, the resulting weights object will iterate
                      over results in the order of the names provided in this
                      argument. 

        See Also
        ---------
        :class:`pysal.weights.W`
        :class:`pysal.weights.Queen`
        """
        idVariable = kwargs.pop('idVariable', None)
        ids = kwargs.pop('ids', None)
        id_order = kwargs.pop('id_order', True)
        if id_order is True and ((idVariable is not None) 
                                  or (ids is not None)):
                # if idVariable is None, we want ids. Otherwise, we want the
                # idVariable column
            ids = list(df.get(idVariable, ids))
            id_order = ids
        elif isinstance(id_order, str):
            ids = df.get(id_order, ids)
            id_order = ids
        elif idVariable is not None:
            ids = df.get(idVariable).tolist()
        elif isinstance(ids, str):
            ids = df.get(ids).tolist()
        else:
            id_order = list(df.index)
            ids = list(df.index)
        w = cls.from_iterable(df[geom_col].tolist(), ids=ids, id_order=id_order, **kwargs)
        return w

def _build(polygons, criterion="rook", ids=None, method='list'):
    """
    This is a developer-facing function to construct a spatial weights object. 

    Arguments
    ---------
    polygons    : list
                  list of pysal polygons to use to build contiguity
    criterion   : string
                  option of which kind of contiguity to build. Is either "rook" or "queen" 
    ids         : list
                  list of ids to use to index the neighbor dictionary
    method      : string
                  option of which driver to use to build contiguity. Three are
                  supported: list, binning, & rtree

    Returns
    -------
    tuple containing (neighbors, ids), where neighbors is a dictionary
    describing contiguity relations and ids is the list of ids used to index
    that dictionary. 

    NOTE: this is different from the prior behavior of buildContiguity, which
          returned an actual weights object. Since this just dispatches for the
          classes above, this returns the raw ingredients for a spatial weights
          object, not the object itself. 
    """
    if ids and len(ids) != len(set(ids)):
        raise ValueError("The argument to the ids parameter contains duplicate entries.")

    wttype = WT_TYPE[criterion.lower()]
    geo = polygons
    if issubclass(type(geo), FileIO):
        geo.seek(0)  # Make sure we read from the beginning of the file.

    if method.lower().startswith('rtree'):
        neighbor_data = ContiguityWeights_rtree(polygons, joinType=wttype).w
    elif method.lower().startswith('list'):
        neighbor_data = ContiguityWeightsLists(polygons, wttype=wttype).w
    else:
        neighbor_data = ContiguityWeightsPolygons(polygons, wttype=wttype).w

    neighbors = {}
    #weights={}
    if ids:
        for key in neighbor_data:
            ida = ids[key]
            if ida not in neighbors:
                neighbors[ida] = set()
            neighbors[ida].update([ids[x] for x in neighbor_data[key]])
        for key in neighbors:
            neighbors[key] = list(neighbors[key])
    else:
        for key in neighbor_data:
            neighbors[key] = list(neighbor_data[key])
    return neighbors, ids

def buildContiguity(polygons, criterion="rook", ids=None):
    """
    This is a deprecated function.

    It builds a contiguity W from the polygons provided. As such, it is now
    identical to calling the class constructors for Rook or Queen. 
    """
    #Warn('This function is deprecated. Please use the Rook or Queen classes',
    #        UserWarning)
    if criterion.lower() == 'rook':
        return Rook(polygons, ids=ids)
    elif criterion.lower() == 'queen':
        return Queen(polygons, ids=ids)
    else:
        raise Exception('Weights criterion "{}" was not found.'.format(criterion))



#!/usr/bin/env python

"""
A library for computing local K function for network-constrained data

Author:
Andrew Winslow Andrew.Winslow@asu.edu
Myunghwa Hwang mhwang4@gmail.com

"""
import unittest
import numpy as np
import geodanet.network as pynet
import geodanet.kfuncs as pykfuncs
import geodanet.simulator as pysim
import time
import random
import platform                                                                                                                 
try:
    if platform.system() == 'Darwin':
        import multiprocessing
    else:
        multiprocessing = None
except ImportError:
    multiprocessing = None 

class WeightedRandomSampleGenerator(object):
    """
    A generator for randomly sampling n elements from 
    a population group with consideration to a given set of weights
    """

    def __init__(self, weights, population, n):
        """
        weights: an iterable with m numeric elements
        population: a numpy array with m elements
        n: an integer representing sample size
        """
        self.totals = np.cumsum(weights)
        self.population = population
        self.n = n
        self.norm = self.totals[-1]

    def next(self):
        sample = []
        for i in xrange(self.n):
            throw = np.random.rand()*self.norm
            sample.append(self.population[np.searchsorted(self.totals, throw)])
        return sample

    def __call__(self):
        return self.next()

class RandomSampleGenerator(object):
    """
    A generator for randomly sampling n elements 
    from a population group
    """
    def __init__(self, population, n):
        """
        population: a numpy array with m elements
        n: an integer representing sample size
        """
        self.population = population
        self.n = n

    def next(self):
        return random.sample(self.population, self.n)

    def __call__(self):
        return self.next()

def local_k(network, events, refs, scale_set, cache=None):
    """
    Computes local K function

    network: an undirected network data to which reference points are injected
    refs: a set of reference points on the given network
          points unprojected into the network
    events: a set of event points on the given network
            points projected into the network
    scale_set: a tuple defining spatial scales to be examined
               (min, max, interval)
    """

    node2localK = {}
    net_distances = {}
    if cache: net_distances = cache
    for node in refs:
        node = node[1][0]
        a_dest = network[node].keys()[0]
        node_proj = (node, a_dest, 0, network[node][a_dest])
        if node not in net_distances:
            net_distances[node] = pynet.dijkstras(network, node, scale_set[1])
        if a_dest not in net_distances:
            net_distances[a_dest] = pynet.dijkstras(network, node, scale_set[1])
        distances = pynet.proj_distances_undirected(network, node_proj, events, scale_set[1], cache=net_distances).values()
        node2localK[node] = pykfuncs.kt_values(scale_set, distances, 1)
    return node2localK, net_distances

def cluster_type(obs, lower, upper):
    if obs < lower: return -1
    if obs > upper: return 1
    return 0

def simulate_local_k_01(args):
    sims = args[0]
    n = args[1]
    net_file = args[2]
    network = args[3]
    events = args[4]
    refs = args[5]
    scale_set = args[6]
    cache = args[7]

    #print 'simulated_local_k_01'
    simulator = pysim.Simulation(net_file)
    sims_outcomes = []
    for sim in xrange(sims):
        points = simulator.getRandomPoints(n, projected=True)
        sim_events = []
        for edge in points:
            for point in points[edge]:
                sim_events.append(point)
        res, dists = local_k(network, sim_events, refs, scale_set, cache=cache)
        sims_outcomes.append(res)

    return sims_outcomes

def simulate_local_k_02(args):
    sims = args[0]
    n = args[1]
    refs = args[2]
    scale_set = args[3]
    cache = args[4]

    #print 'simulated_local_k_02'
    sims_outcomes = []
    sampler = RandomSampleGenerator(refs, n).next
    for sim in xrange(sims):
        sim_events = sampler()
        sim_localk = {}
        for node in refs:
            all_distances = cache[node[1][0]]
            distances = []
            for event in sim_events:
                event = event[1][0]
                if event in all_distances:
                    distances.append(all_distances[event]) 
            sim_localk[node[1][0]] = pykfuncs.kt_values(scale_set, distances, 1)
        sims_outcomes.append(sim_localk)
    
    return sims_outcomes

def k_cluster(network, events, refs, scale_set, sims, sig=0.1, sim_network=None, cpus=1):

    """
    Parameters:
    network: a network to which reference points are injected
    events: a set of event points projected into the network
    refs: a set of reference points unprojected into the network
    scale_set: tuple same as (min, max, resolution)
    sims: integer; the number of simulations
    sig: float; siginificance level
    sim_network: the source shape file containing the network data
                 this is used to simualte point patterns for inference 
    cpus: integer: the number of cpus
          multiprocessing can be used for inference
    """

    """
    1. For an observed set of n events on the network, calculate local K function 
    values for all m reference points
    """
    node2localK, net_dists = local_k(network, events, refs, scale_set)
    """
    When n < m (simulator == None):
    2. Select n out of m reference points randomly and 
    calculate local K function values for these randomly sampled points
    When n >= m (simulator != None):
    2. Randomly simulate n points on network edges and
    calculate local K function values for these randomly simulated points
    3. Repeat 2 as many as the number of simulations
    Note: on Darwin systems, simulation will be parallelized
    """
    n = len(events)
    sims_outcomes = []
    if not multiprocessing or cpus == 1:
        if sim_network:
            sims_outcomes = simulate_local_k_01((sims, n, sim_network, network, events, refs, scale_set, net_dists))
        else:
            sims_outcomes = simulate_local_k_02((sims, n, refs, scale_set, net_dists))
    elif multiprocessing and cpus >= 2:
        pool = multiprocessing.Pool(cpus)
        sims_list = range(sims)
        sims_list = map(len, [sims_list[i::cpus] for i in xrange(cpus)])
        partial_outcomes = None
        if sim_network:
             partial_outcomes = pool.map(simulate_local_k_01, 
                         [(sim, n, sim_network, network, events, refs, scale_set, net_dists) for sim in sims_list])
        else:
             partial_outcomes = pool.map(simulate_local_k_02, 
                         [(sim, n, refs, scale_set, net_dists) for sim in sims_list])
        sims_outcomes = partial_outcomes[0]
        for partial in partial_outcomes[1:]:
             sims_outcomes.extend(partial)

    """
    4. Determine lower and upper envelopes for the observed K function values 
       as well as the type of cluster (dispersion or clustering)
    """
    # 4. P-value evaluation
    lower_envelope = {}
    upper_envelope = {}
    lower_p = int(sims*sig/2)
    upper_p = int(sims*(1-sig/2))
    localKs = {}
    for node in refs:
        node = node[1][0]
        lower_envelope[node] = {}
        upper_envelope[node] = {}
        localKs[node] = {}
        for scale in node2localK[node].keys():
            local_outcomes = [sim[node][scale] for sim in sims_outcomes]
            local_outcomes.sort()
            obs = node2localK[node][scale]
            lower = local_outcomes[lower_p]
            upper = local_outcomes[upper_p]
            cluster = cluster_type(obs, lower, upper)
            localKs[node][scale] = [obs, lower, upper, cluster]

    return localKs




#from ...common import requires as _requires

import itertools as _it
from pysal.weights import W

# I would like to define it like this, so that you could make a call like:
# Geary(df, 'HOVAL', 'INC', w=W), but this only works in Python3. So, I have to
# use a workaround
#def _statistic(df, *cols, stat=None, w=None, inplace=True, 
def _univariate_handler(df, cols, stat=None, w=None, inplace=True,
                        pvalue = 'sim', outvals = None, swapname='', **kwargs):
    """
    Compute a univariate descriptive statistic `stat` over columns `cols` in
    `df`.

    Parameters
    ----------
    df          : pandas.DataFrame
                  the dataframe containing columns to compute the descriptive
                  statistics
    cols        : string or list of strings
                  one or more names of columns in `df` to use to compute
                  exploratory descriptive statistics. 
    stat        : callable
                  a function that takes data as a first argument and any number
                  of configuration keyword arguments and returns an object
                  encapsulating the exploratory statistic results 
    w           : pysal.weights.W
                  the spatial weights object corresponding to the dataframe
    inplace     : bool
                  a flag denoting whether to add the statistic to the dataframe
                  in memory, or to construct a copy of the dataframe and append
                  the results to the copy
    pvalue      : string
                  the name of the pvalue on the results object wanted
    outvals     : list of strings
                  names of attributes of the dataframe to attempt to flatten
                  into a column
    swapname    : string
                  suffix to replace generic identifier with. Each caller of this
                  function should set this to a unique column suffix
    **kwargs    : optional keyword arguments
                  options that are passed directly to the statistic
    """
    ### Preprocess
    if not inplace:
        new_df = df.copy()
        _univariate_handler(new_df, cols, stat=stat, w=w, pvalue=pvalue, 
                                   inplace=True, outvals=outvals,
                                   swapname=swapname, **kwargs)
        return new_df
    if w is None:
        for name in df._metadata:
            this_obj = df.__dict__.get(name)
            if isinstance(this_obj, W):
                w = this_obj
    if w is None:
        raise Exception('Weights not provided and no weights attached to frame!'
                            ' Please provide a weight or attach a weight to the'
                            ' dataframe')
    ### Prep indexes
    if outvals is None:
        outvals = []
    outvals.insert(0,'_statistic')
    if pvalue.lower() in ['all', 'both', '*']: 
        raise NotImplementedError("If you want more than one type of PValue,add"
                                  " the targeted pvalue type to outvals. For example:"
                                  " Geary(df, cols=['HOVAL'], w=w, outvals=['p_z_sim', "
                                  "'p_rand']")
    # this is nontrivial, since we
    # can't know which p_value types are on the object without computing it.
    # This is because we don't flag them with @properties, so they're just
    # arbitrarily assigned post-facto. One solution might be to post-process the
    # objects, determine which pvalue types are available, and then grab them
    # all if needed. 
    
    if pvalue is not '':
        outvals.append('p_'+pvalue.lower())
    if isinstance(cols, str):
        cols = [cols]

    ### Make closure around weights & apply columnwise
    def column_stat(column):
        return stat(column.values, w=w, **kwargs)
    stat_objs = df[cols].apply(column_stat)

    ### Assign into dataframe
    for col in cols:
        stat_obj = stat_objs[col]
        y = kwargs.get('y')
        if y is not None:
            col += '-' + y.name
        outcols = ['_'.join((col, val)) for val in outvals]
        for colname, attname in zip(outcols, outvals):
            df[colname] = stat_obj.__getattribute__(attname)
    if swapname is not '':
        df.columns = [_swap_ending(col, swapname) if col.endswith('_statistic') else col
                      for col in df.columns]

def _bivariate_handler(df, x, y=None, w=None, inplace=True, pvalue='sim', 
                       outvals=None, **kwargs):
    """
    Compute a descriptive bivariate statistic over two sets of columns, `x` and
    `y`, contained in `df`. 

    Parameters
    ----------
    df          : pandas.DataFrame
                  dataframe in which columns `x` and `y` are contained
    x           : string or list of strings
                  one or more column names to use as variates in the bivariate
                  statistics
    y           : string or list of strings
                  one or more column names to use as variates in the bivariate
                  statistics
    w           : pysal.weights.W
                  spatial weights object corresponding to the dataframe `df`
    inplace     : bool
                  a flag denoting whether to add the statistic to the dataframe
                  in memory, or to construct a copy of the dataframe and append
                  the results to the copy
    pvalue      : string
                  the name of the pvalue on the results object wanted
    outvals     : list of strings
                  names of attributes of the dataframe to attempt to flatten
                  into a column
    swapname    : string
                  suffix to replace generic identifier with. Each caller of this
                  function should set this to a unique column suffix
    **kwargs    : optional keyword arguments
                  options that are passed directly to the statistic
    """
    real_swapname = kwargs.pop('swapname', '')
    if isinstance(y, str):
        y = [y]
    if isinstance(x, str):
        x = [x]
    if not inplace:
        new_df = df.copy()
        _bivariate_handler(new_df, x, y=y, w=w, inplace=True,
                           swapname=real_swapname, 
                           pvalue=pvalue, outvals=outvals, **kwargs)
        return new_df
    if y is None:
        y = x
    for xi,yi in _it.product(x,y):
        if xi == yi:
            continue
        _univariate_handler(df, cols=xi, w=w, y=df[yi], inplace=True, 
                            pvalue=pvalue, outvals=outvals, swapname='', **kwargs)
    if real_swapname is not '':
        df.columns = [_swap_ending(col, real_swapname) 
                      if col.endswith('_statistic')
                      else col for col in df.columns]

def _swap_ending(s, ending, delim='_'):
    """
    Replace the ending of a string, delimited into an arbitrary 
    number of chunks by `delim`, with the ending provided

    Parameters
    ----------
    s       :   string
                string to replace endings
    ending  :   string
                string used to replace ending of `s`
    delim   :   string
                string that splits s into one or more parts
    
    Returns
    -------
    new string where the final chunk of `s`, delimited by `delim`, is replaced
    with `ending`. 
    """
    parts = [x for x in s.split(delim)[:-1] if x != '']
    parts.append(ending)
    return delim.join(parts)
 
##############
# DOCSTRINGS #
##############

_univ_doc_template =\
""" 
Function to compute a {n} statistic on a dataframe

Arguments
---------
df          :   pandas.DataFrame
                a pandas dataframe with a geometry column
cols        :   string or list of string
                name or list of names of columns to use to compute the statistic
w           :   pysal weights object
                a weights object aligned with the dataframe. If not provided, this
                is searched for in the dataframe's metadata
inplace     :   bool
                a boolean denoting whether to operate on the dataframe inplace or to
                return a series contaning the results of the computation. If
                operating inplace, the derived columns will be named 'column_{nl}'
pvalue      :   string
                a string denoting which pvalue should be returned. Refer to the
                the {n} statistic's documentation for available p-values
outvals     :   list of strings
                list of arbitrary attributes to return as columns from the 
                {n} statistic
**stat_kws  :   keyword arguments
                options to pass to the underlying statistic. For this, see the
                documentation for the {n} statistic.

Returns
--------
If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
returns a copy of the dataframe with the relevant columns attached.

See Also
---------
For further documentation, refer to the {n} class in pysal.esda
"""

_bv_doc_template =\
""" 
Function to compute a {n} statistic on a dataframe

Arguments
---------
df          :   pandas.DataFrame
                a pandas dataframe with a geometry column
X           :   list of strings
                column name or list of column names to use as X values to compute
                the bivariate statistic. If no Y is provided, pairwise comparisons
                among these variates are used instead. 
Y           :   list of strings
                column name or list of column names to use as Y values to compute
                the bivariate statistic. if no Y is provided, pariwise comparisons
                among the X variates are used instead. 

w           :   pysal weights object
                a weights object aligned with the dataframe. If not provided, this
                is searched for in the dataframe's metadata
inplace     :   bool
                a boolean denoting whether to operate on the dataframe inplace or to
                return a series contaning the results of the computation. If
                operating inplace, the derived columns will be named 'column_{nl}'
pvalue      :   string
                a string denoting which pvalue should be returned. Refer to the
                the {n} statistic's documentation for available p-values
outvals     :   list of strings
                list of arbitrary attributes to return as columns from the 
                {n} statistic
**stat_kws  :   keyword arguments
                options to pass to the underlying statistic. For this, see the
                documentation for the {n} statistic.


Returns
--------
If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
returns a copy of the dataframe with the relevant columns attached.

See Also
---------
For further documentation, refer to the {n} class in pysal.esda
"""

_rate_doc_template =\
""" 
Function to compute a {n} statistic on a dataframe

Arguments
---------
df          :   pandas.DataFrame
                a pandas dataframe with a geometry column
events      :   string or  list of strings
                one or more names where events are stored
populations :   string or list of strings
                one or more names where the populations corresponding to the
                events are stored. If one population column is provided, it is
                used for all event columns. If more than one population column
                is provided but there is not a population for every event
                column, an exception will be raised.
w           :   pysal weights object
                a weights object aligned with the dataframe. If not provided, this
                is searched for in the dataframe's metadata
inplace     :   bool
                a boolean denoting whether to operate on the dataframe inplace or to
                return a series contaning the results of the computation. If
                operating inplace, the derived columns will be named 'column_{nl}'
pvalue      :   string
                a string denoting which pvalue should be returned. Refer to the
                the {n} statistic's documentation for available p-values
outvals     :   list of strings
                list of arbitrary attributes to return as columns from the 
                {n} statistic
**stat_kws  :   keyword arguments
                options to pass to the underlying statistic. For this, see the
                documentation for the {n} statistic.

Returns
--------
If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
returns a copy of the dataframe with the relevant columns attached.

See Also
---------
For further documentation, refer to the {n} class in pysal.esda
"""



"""
cleanNetShp -- Tools to clean spatial Network Shapefiles.
"""
import pysal
import numpy

__author__ = "Charles R. Schmidt <schmidtc@gmail.com>"
__all__ = ['snap_verts', 'find_nodes', 'split_at_nodes']


def snap_verts(shp,tolerance=0.001,arc=True):
    """
    snap_verts -- Snap verts that are within tolerance meters of each other.

    Description -- Snapping should be performed with a very small tolerance.
                   The goal is not to change the network, but to ensure rounding
                   errors don't prevent edges from being split at proper intersections.
                   The default of 1mm should be adequate if the input is of decent quality.
                   Higher snapping values can be used to correct digitizing errors, but care
                   should be taken.

    Arguments
    ---------
    tolerance -- float -- snapping tolerance in meters
    arc -- bool -- If true, Ard Distance will be used instead of Euclidean

    Returns
    -------
    generator -- each element is a new pysal.cg.Chain with corrected vertices.
    """
    kmtol = tolerance/1000.

    data = numpy.concatenate([rec.vertices for rec in shp])
    
    if arc:
        kd = pysal.cg.KDTree(data,distance_metric="Arc",radius = pysal.cg.sphere.RADIUS_EARTH_KM)
    else:
        kd = pysal.cg.KDTree(data)
    q = kd.query_ball_tree(kd,kmtol)
    ### Next three lines assert that snappings are mutual... if 1 snaps to 8, 8 must snap to 1.
    for r,a in enumerate(q):
        for o in a:
            assert a==q[o]
    ### non-mutual snapping can happen.
    ### consider the three points, A (-1,0), B (0,0), C (1,0) and a snapping tolerance of 1.
    ### A-> B
    ### B-> A,C
    ### C-> B
    ### For now, try lowering adjusting the tolerance to avoid this.

    data2 = numpy.empty_like(data)
    for i,r in enumerate(q):
        data2[i] = data[r].mean(0)
    pos=0
    for rec in shp:
        vrts = rec.vertices
        n = len(vrts)
        nrec = pysal.cg.Chain(map(tuple,data2[pos:pos+n]))
        pos+=n
        yield nrec
    
def find_nodes(shp):
    """
    find_nodes -- Finds vertices in a line type shapefile that appear more than once and/or are end points of a line

    Arguments
    ---------
    shp -- Shapefile Object -- Should be of type Line.

    Returns
    -------
    set
    """
    node_count = {}
    for road in shp:
        vrts = road.vertices
        for node in vrts:
            if node not in node_count:
                node_count[node] = 0
            node_count[node] += 1
        node_count[vrts[0]] += 1
        node_count[vrts[-1]] += 1
    return set([node for node,c in node_count.iteritems() if c > 1])

def split_at_nodes(shp):
    """
    split_at_nodes -- Split line features at nodes

    Arguments
    ---------
    shp -- list or shapefile -- Chain features to be split at common nodes.

    Returns
    -------
    generator -- yields pysal.cg.Chain objects
    """
    nodes = find_nodes(shp)
    nodeIds = list(nodes)
    nodeIds.sort()
    nodeIds = dict([(node,i) for i,node in enumerate(nodeIds)])
    
    for road in shp:
        vrts = road.vertices
        midVrts = set(road.vertices[1:-1]) #we know end points are nodes
        midNodes = midVrts.intersection(nodes) # find any nodes in the middle of the feature.
        midIdx = [vrts.index(node) for node in midNodes] # Get their indices
        midIdx.sort()
        if midIdx:
            #print vrts
            starts = [0]+midIdx
            stops = [x+1 for x in midIdx]+[None]
            for start,stop in zip(starts,stops):
                feat = pysal.cg.Chain(vrts[start:stop])
                rec = (nodeIds[feat.vertices[0]],nodeIds[feat.vertices[-1]],False)
                yield feat,rec
        else:
            rec = (nodeIds[road.vertices[0]],nodeIds[road.vertices[-1]],False)
            yield road,rec


def createSpatialNetworkShapefile(inshp,outshp):
    assert inshp.lower().endswith('.shp')
    assert outshp.lower().endswith('.shp')
    shp = pysal.open(inshp,'r')
    snapped = list(snap_verts(shp,.001))
    o = pysal.open(outshp,'w')
    odb = pysal.open(outshp[:-4]+'.dbf','w')
    odb.header = ["FNODE","TNODE","ONEWAY"]
    odb.field_spec = [('N',20,0),('N',20,0),('L',1,0)]

    new = list(split_at_nodes(snapped))
    for feat,rec in new:
        o.write(feat)
        odb.write(rec)
    o.close()
    odb.close()
    print "Split %d roads in %d network edges"%(len(shp),len(new))

if __name__=='__main__':
    createSpatialNetworkShapefile('beth_roads.shp','beth_network.shp')




"""network unittest"""
import unittest
import network as pynet
import lincs 
import random
random.seed(10)
import pysal
import numpy as np
import copy

class LINCS_Tester(unittest.TestCase):

    def setUp(self):
        self.base = np.array([100, 80, 50, 120, 90])
        self.events = np.array([20, 20, 5, 10, 25])
        np.random.seed(10)
        self.observed = np.array([0.1,0.15,0.2])
        self.simulated = np.array([[0.05,0.10,0.25],[0.12,0.11,0.3],[0.11,0.09,0.27]])
        self.network_file = 'streets.shp'
        self.G = pynet.read_network(self.network_file)
        self.test_link = ((724432.38723173144, 877800.08747069736), 
                          (724587.78057580709, 877802.4281426128))
        self.G2 = copy.deepcopy(self.G)
        done = set()
        for n1 in self.G2:
            for n2 in self.G2[n1]:
                if (n1, n2) in done:
                    continue
                dist = self.G2[n1][n2]
                base = int(random.random()*1000)
                event = int(random.random()*base)
                self.G2[n1][n2] = [dist, base, event]
                self.G2[n2][n1] = [dist, base, event]
                done.add((n1,n2))
                done.add((n2,n1))

    def test_unconditional_sim(self):
        simulated_events = lincs.unconditional_sim(self.events, self.base, 2)
        self.assertEqual(list(simulated_events[0]),[21,15])

    def test_unconditional_sim_poisson(self):
        simulated_events = lincs.unconditional_sim_poisson(self.events, self.base, 2)
        self.assertEqual(list(simulated_events[0]),[22,21])

    def test_conditional_multinomial(self):
        simulated_events = lincs.conditional_multinomial(self.events, self.base, 2)
        self.assertEqual(list(simulated_events[0]),[21,18])

    def test_pseudo_pvalues(self):
        pseudo_pvalues = lincs.pseudo_pvalues(self.observed, self.simulated)
        self.assertEqual(list(pseudo_pvalues[0]),[ 0.5,  0.5,  0.5])

    def test_node_weights(self):
        w, id2link = lincs.node_weights(self.G)
        self.assertEqual(w.neighbors[0],[1, 2, 3, 4])

    def test_edgepoints_from_network(self):
        id2linkpoints, id2attr, link2id = lincs.edgepoints_from_network(self.G)
        link = id2linkpoints[0]
        self.assertEqual(link[:2], self.test_link)
        self.assertEqual(link2id[link[:2]], 0)

    def test_dist_weights(self):
        id2linkpoints, id2attr, link2id = lincs.edgepoints_from_network(self.G)
        w, id2link = lincs.dist_weights(self.G, id2linkpoints, link2id, 500)
        self.assertEqual(w.neighbors[0],[1,154,153,155])
        self.assertEqual(id2link[0], self.test_link)

    def test_lincs(self):
        network = self.G2
        event_index = 2
        base_index = 1
        weight = 'Distance-based'
        dist = 500
        lisa_func = 'moran'
        sim_method = 'permutations'
        sim_num = 2
        lisa, w = lincs.lincs(network, event_index, base_index, weight, dist, lisa_func, sim_method, sim_num)
        self.assertEqual(lisa[0][3], -0.64342055427251854)

suite = unittest.TestSuite()
test_classes = [LINCS_Tester]
for i in test_classes:
    a = unittest.TestLoader().loadTestsFromTestCase(i)
    suite.addTest(a)

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite)



"""
Utility module for network contrib 
"""

import pysal as ps
import networkx as nx
import numpy as np

__author__ = "Serge Rey <sjsrey@gmail.com>"

def w2dg(w):
    """
    Return a networkx directed graph from a PySAL W object


    Parameters
    ----------

    w: Weights 


    Returns
    -------
    G: A networkx directed graph


    Example
    ------

    >>> import networkx as nx
    >>> import pysal as ps
    >>> w = ps.lat2W()
    >>> guw = w2dg(w)
    >>> guw.in_degree()
    {0: 2, 1: 3, 2: 3, 3: 3, 4: 2, 5: 3, 6: 4, 7: 4, 8: 4, 9: 3, 10: 3, 11: 4, 12: 4, 13: 4, 14: 3, 15: 3, 16: 4, 17: 4, 18: 4, 19: 3, 20: 2, 21: 3, 22: 3, 23: 3, 24: 2}
    >>> dict([(k,len(w.neighbors[k])) for k in w.neighbors])
    {0: 2, 1: 3, 2: 3, 3: 3, 4: 2, 5: 3, 6: 4, 7: 4, 8: 4, 9: 3, 10: 3, 11: 4, 12: 4, 13: 4, 14: 3, 15: 3, 16: 4, 17: 4, 18: 4, 19: 3, 20: 2, 21: 3, 22: 3, 23: 3, 24: 2}
    >>> 
    """

    w_l = [(i,j) for i in w.neighbors for j in w[i]]
    G = nx.DiGraph()
    G.add_edges_from(w_l)
    return G

def w2dwg(w):
    """
    Return a directed, weighted graph from a PySAL W object


    Parameters
    ----------

    w: Weights 


    Returns
    -------
    G: A networkx directed, weighted graph


    Example
    -------
    >>> import networkx as nx
    >>> import pysal as ps
    >>> w = ps.lat2W()
    >>> w.transform = 'r'
    >>> gw = w2dwg(w)
    >>> gw.get_edge_data(0,1)
    {'weight': 0.5}
    >>> gw.get_edge_data(1,0)
    {'weight': 0.33333333333333331}
    """

    w_l = [(i,j,w[i][j]) for i in w.neighbors for j in w[i]]
    G = nx.DiGraph() # allow for asymmetries in weights
    G.add_weighted_edges_from(w_l)
    return G


def dwg2w(g, weight_name = 'weight'):
    """
    Returns a PySAL W object from a directed-weighted graph

    Parameters
    ----------

    g: networkx digraph

    weight_name: name of weight attribute of g

    Returns
    -------
    w: PySAL W 

    Example
    -------
    >>> w = ps.lat2W()
    >>> w.transform = 'r'
    >>> g = w2dwg(w)
    >>> w1 = dwg2w(g)
    >>> w1.n
    25
    >>> w1.neighbors[0]
    [1, 5]
    >>> w1.neighbors[1]
    [0, 2, 6]
    >>> w1.weights[0]
    [0.5, 0.5]
    >>> w1.weights[1]
    [0.33333333333333331, 0.33333333333333331, 0.33333333333333331]
    """

    neighbors = {}
    weights = {}
    for node in g.nodes_iter():
        neighbors[node] = []
        weights[node] = []
        for neighbor in g.neighbors_iter(node):
            neighbors[node].append(neighbor)
            weight = g.get_edge_data(node,neighbor)
            if weight:
                weights[node].append(weight[weight_name])
            else:
                weights[node].append(1)
    return ps.W(neighbors=neighbors, weights=weights)




def edge2w(edgelist, nodetype=str):
    """
    Create a PySAL W object from an edgelist

    Parameters
    ----------

    edge_file: file with edgelist

    nodetype: type for node (str, int, float)


    Returns
    -------
    W: PySAL W


    Example
    -------
    >>> lines = ["1 2", "2 3", "3 4", "4 5"]
    >>> w = edge2w(lines)
    >>> w.n
    5
    >>> w.neighbors["2"]
    ['1', '3']

    >>> w = edge2w(lines, nodetype=int)
    >>> w.neighbors[2]
    [1, 3]
    >>> lines = ["1 2 {'weight':1.0}", "2 3 {'weight':0.5}", "3 4 {'weight':3.0}"] 
    >>> w = edge2w(lines, nodetype=int)
    >>> w.neighbors[2]
    [1, 3]
    >>> w.weights[2]
    [1.0, 0.5]

    """
    G = nx.parse_edgelist(edgelist, nodetype=nodetype)
    return dwg2w(G)

    

def adjl2w(adjacency_list, nodetype=str):
    """
    Create a PySAL W object from an adjacency list file

    Parameters
    ----------

    adjacency_list: list of adjacencies
                    for directed graphs list only outgoing adjacencies

    nodetype: type for node (str, int, float)


    Returns
    -------
    W: PySAL W


    Example
    -------
    >>> al = [[1], [0,2], [1,3], [2]]
    >>> w = adjl2w(al)
    >>> w.n
    4
    >>> w.neighbors['0']
    ['1']
    >>> w = adjl2w(al, nodetype=int)
    >>> w.n
    4
    >>> w.neighbors[0]
    [1]


    """

    adjacency_list = [ map(nodetype, neighs) for neighs in adjacency_list]
    return ps.W(dict([(nodetype(i),neighs) for i,neighs in enumerate(adjacency_list)]))

                       

if __name__ == '__main__':
    import doctest
    doctest.testmod()










from __future__ import print_function
import numpy as np
from scipy import stats
from utils import cache_readonly

class Results(object):
    """
    Class to contain model results
    Parameters
    ----------
    model : class instance
        the previously specified model instance
    params : array
        parameter estimates from the fit model
    """
    def __init__(self, model, params, **kwd):
        self.__dict__.update(kwd)
        self.initialize(model, params, **kwd)
        self._data_attr = []

    def initialize(self, model, params, **kwd):
        self.params = params
        self.model = model
        if hasattr(model, 'k_constant'):
            self.k_constant = model.k_constant

#TODO: public method?
class LikelihoodModelResults(Results):
    """
    Class to contain results from likelihood models
    Parameters
    -----------
    model : LikelihoodModel instance or subclass instance
        LikelihoodModelResults holds a reference to the model that is fit.
    params : 1d array_like
        parameter estimates from estimated model
    normalized_cov_params : 2d array
       Normalized (before scaling) covariance of params. (dot(X.T,X))**-1
    scale : float
        For (some subset of models) scale will typically be the
        mean square error from the estimated model (sigma^2)
    Returns
    -------
    **Attributes**
    mle_retvals : dict
        Contains the values returned from the chosen optimization method if
        full_output is True during the fit.  Available only if the model
        is fit by maximum likelihood.  See notes below for the output from
        the different methods.
    mle_settings : dict
        Contains the arguments passed to the chosen optimization method.
        Available if the model is fit by maximum likelihood.  See
        LikelihoodModel.fit for more information.
    model : model instance
        LikelihoodResults contains a reference to the model that is fit.
    params : ndarray
        The parameters estimated for the model.
    scale : float
        The scaling factor of the model given during instantiation.
    tvalues : array
        The t-values of the standard errors.
    Notes
    -----
    The covariance of params is given by scale times normalized_cov_params.
    Return values by solver if full_output is True during fit:
        'newton'
            fopt : float
                The value of the (negative) loglikelihood at its
                minimum.
            iterations : int
                Number of iterations performed.
            score : ndarray
                The score vector at the optimum.
            Hessian : ndarray
                The Hessian at the optimum.
            warnflag : int
                1 if maxiter is exceeded. 0 if successful convergence.
            converged : bool
                True: converged. False: did not converge.
            allvecs : list
                List of solutions at each iteration.
        'nm'
            fopt : float
                The value of the (negative) loglikelihood at its
                minimum.
            iterations : int
                Number of iterations performed.
            warnflag : int
                1: Maximum number of function evaluations made.
                2: Maximum number of iterations reached.
            converged : bool
                True: converged. False: did not converge.
            allvecs : list
                List of solutions at each iteration.
        'bfgs'
            fopt : float
                Value of the (negative) loglikelihood at its minimum.
            gopt : float
                Value of gradient at minimum, which should be near 0.
            Hinv : ndarray
                value of the inverse Hessian matrix at minimum.  Note
                that this is just an approximation and will often be
                different from the value of the analytic Hessian.
            fcalls : int
                Number of calls to loglike.
            gcalls : int
                Number of calls to gradient/score.
            warnflag : int
                1: Maximum number of iterations exceeded. 2: Gradient
                and/or function calls are not changing.
            converged : bool
                True: converged.  False: did not converge.
            allvecs : list
                Results at each iteration.
        'lbfgs'
            fopt : float
                Value of the (negative) loglikelihood at its minimum.
            gopt : float
                Value of gradient at minimum, which should be near 0.
            fcalls : int
                Number of calls to loglike.
            warnflag : int
                Warning flag:
                - 0 if converged
                - 1 if too many function evaluations or too many iterations
                - 2 if stopped for another reason
            converged : bool
                True: converged.  False: did not converge.
        'powell'
            fopt : float
                Value of the (negative) loglikelihood at its minimum.
            direc : ndarray
                Current direction set.
            iterations : int
                Number of iterations performed.
            fcalls : int
                Number of calls to loglike.
            warnflag : int
                1: Maximum number of function evaluations. 2: Maximum number
                of iterations.
            converged : bool
                True : converged. False: did not converge.
            allvecs : list
                Results at each iteration.
        'cg'
            fopt : float
                Value of the (negative) loglikelihood at its minimum.
            fcalls : int
                Number of calls to loglike.
            gcalls : int
                Number of calls to gradient/score.
            warnflag : int
                1: Maximum number of iterations exceeded. 2: Gradient and/
                or function calls not changing.
            converged : bool
                True: converged. False: did not converge.
            allvecs : list
                Results at each iteration.
        'ncg'
            fopt : float
                Value of the (negative) loglikelihood at its minimum.
            fcalls : int
                Number of calls to loglike.
            gcalls : int
                Number of calls to gradient/score.
            hcalls : int
                Number of calls to hessian.
            warnflag : int
                1: Maximum number of iterations exceeded.
            converged : bool
                True: converged. False: did not converge.
            allvecs : list
                Results at each iteration.
        """

    # by default we use normal distribution
    # can be overwritten by instances or subclasses
    use_t = False

    def __init__(self, model, params, normalized_cov_params=None, scale=1.,
                 **kwargs):
        super(LikelihoodModelResults, self).__init__(model, params)
        self.normalized_cov_params = normalized_cov_params
        self.scale = scale

        # robust covariance
        # We put cov_type in kwargs so subclasses can decide in fit whether to
        # use this generic implementation
        if 'use_t' in kwargs:
            use_t = kwargs['use_t']
            if use_t is not None:
                self.use_t = use_t
        if 'cov_type' in kwargs:
            cov_type = kwargs.get('cov_type', 'nonrobust')
            cov_kwds = kwargs.get('cov_kwds', {})

            if cov_type == 'nonrobust':
                self.cov_type = 'nonrobust'
                self.cov_kwds = {'description' : 'Standard Errors assume that the ' +
                                 'covariance matrix of the errors is correctly ' +
                                 'specified.'}
            else:
                from statsmodels.base.covtype import get_robustcov_results
                if cov_kwds is None:
                    cov_kwds = {}
                use_t = self.use_t
                # TODO: we shouldn't need use_t in get_robustcov_results
                get_robustcov_results(self, cov_type=cov_type, use_self=True,
                                           use_t=use_t, **cov_kwds)


    def normalized_cov_params(self):
        raise NotImplementedError


    def _get_robustcov_results(self, cov_type='nonrobust', use_self=True,
                                   use_t=None, **cov_kwds):
        from statsmodels.base.covtype import get_robustcov_results
        if cov_kwds is None:
            cov_kwds = {}

        if cov_type == 'nonrobust':
            self.cov_type = 'nonrobust'
            self.cov_kwds = {'description' : 'Standard Errors assume that the ' +
                             'covariance matrix of the errors is correctly ' +
                             'specified.'}
        else:
            # TODO: we shouldn't need use_t in get_robustcov_results
            get_robustcov_results(self, cov_type=cov_type, use_self=True,
                                       use_t=use_t, **cov_kwds)

    @cache_readonly
    def llf(self):
        return self.model.loglike(self.params)

    @cache_readonly
    def bse(self):
        return np.sqrt(np.diag(self.cov_params()))

    @cache_readonly
    def tvalues(self):
        """
        Return the t-statistic for a given parameter estimate.
        """
        return self.params / self.bse

    @cache_readonly
    def pvalues(self):
        if self.use_t:
            df_resid = getattr(self, 'df_resid_inference', self.df_resid)
            return stats.t.sf(np.abs(self.tvalues), df_resid)*2
        else:
            return stats.norm.sf(np.abs(self.tvalues))*2


    def cov_params(self, r_matrix=None, column=None, scale=None, cov_p=None,
            other=None):
        """
        Returns the variance/covariance matrix.
        The variance/covariance matrix can be of a linear contrast
        of the estimates of params or all params multiplied by scale which
        will usually be an estimate of sigma^2.  Scale is assumed to be
        a scalar.
        Parameters
        ----------
        r_matrix : array-like
            Can be 1d, or 2d.  Can be used alone or with other.
        column :  array-like, optional
            Must be used on its own.  Can be 0d or 1d see below.
        scale : float, optional
            Can be specified or not.  Default is None, which means that
            the scale argument is taken from the model.
        other : array-like, optional
            Can be used when r_matrix is specified.
        Returns
        -------
        cov : ndarray
            covariance matrix of the parameter estimates or of linear
            combination of parameter estimates. See Notes.
        Notes
        -----
        (The below are assumed to be in matrix notation.)
        If no argument is specified returns the covariance matrix of a model
        ``(scale)*(X.T X)^(-1)``
        If contrast is specified it pre and post-multiplies as follows
        ``(scale) * r_matrix (X.T X)^(-1) r_matrix.T``
        If contrast and other are specified returns
        ``(scale) * r_matrix (X.T X)^(-1) other.T``
        If column is specified returns
        ``(scale) * (X.T X)^(-1)[column,column]`` if column is 0d
        OR
        ``(scale) * (X.T X)^(-1)[column][:,column]`` if column is 1d
        """
        if (hasattr(self, 'mle_settings') and
                self.mle_settings['optimizer'] in ['l1', 'l1_cvxopt_cp']):
            dot_fun = nan_dot
        else:
            dot_fun = np.dot

        if (cov_p is None and self.normalized_cov_params is None and
            not hasattr(self, 'cov_params_default')):
            raise ValueError('need covariance of parameters for computing '
                             '(unnormalized) covariances')
        if column is not None and (r_matrix is not None or other is not None):
            raise ValueError('Column should be specified without other '
                             'arguments.')
        if other is not None and r_matrix is None:
            raise ValueError('other can only be specified with r_matrix')

        if cov_p is None:
            if hasattr(self, 'cov_params_default'):
                cov_p = self.cov_params_default
            else:
                if scale is None:
                    scale = self.scale
                cov_p = self.normalized_cov_params * scale

        if column is not None:
            column = np.asarray(column)
            if column.shape == ():
                return cov_p[column, column]
            else:
                #return cov_p[column][:, column]
                return cov_p[column[:, None], column]
        elif r_matrix is not None:
            r_matrix = np.asarray(r_matrix)
            if r_matrix.shape == ():
                raise ValueError("r_matrix should be 1d or 2d")
            if other is None:
                other = r_matrix
            else:
                other = np.asarray(other)
            tmp = dot_fun(r_matrix, dot_fun(cov_p, np.transpose(other)))
            return tmp
        else:  # if r_matrix is None and column is None:
            return cov_p

    def conf_int(self, alpha=.05, cols=None, method='default'):
        """
        Returns the confidence interval of the fitted parameters.
        Parameters
        ----------
        alpha : float, optional
            The significance level for the confidence interval.
            ie., The default `alpha` = .05 returns a 95% confidence interval.
        cols : array-like, optional
            `cols` specifies which confidence intervals to return
        method : string
            Not Implemented Yet
            Method to estimate the confidence_interval.
            "Default" : uses self.bse which is based on inverse Hessian for MLE
            "hjjh" :
            "jac" :
            "boot-bse"
            "boot_quant"
            "profile"
        Returns
        --------
        conf_int : array
            Each row contains [lower, upper] limits of the confidence interval
            for the corresponding parameter. The first column contains all
            lower, the second column contains all upper limits.
        Examples
        --------
        >>> import pysal
        >>> from pysal.contrib.glm.glm import GLM
        >>> import numpy as np
        >>> db = pysal.open(pysal.examples.get_path('columbus.dbf'),'r')
        >>> y = np.array(db.by_col("HOVAL")).reshape((-1,1))
        >>> X = []
        >>> X.append(db.by_col("INC"))
        >>> X.append(db.by_col("CRIME"))
        >>> X = np.array(X).T
        >>> model = GLM(y, X)
        >>> results = model.fit()
        >>> results.conf_int()
        array([[ 20.57281401,  72.28355135],
               [ -0.42138121,   1.67934915],
               [ -0.84292086,  -0.12685622]])

        Notes
        -----
        The confidence interval is based on the standard normal distribution.
        Models wish to use a different distribution should overwrite this
        method.
        """
        bse = self.bse

        if self.use_t:
            dist = stats.t
            df_resid = getattr(self, 'df_resid_inference', self.df_resid)
            q = dist.ppf(1 - alpha / 2, df_resid)
        else:
            dist = stats.norm
            q = dist.ppf(1 - alpha / 2)

        if cols is None:
            lower = self.params - q * bse
            upper = self.params + q * bse
        else:
            cols = np.asarray(cols)
            lower = self.params[cols] - q * bse[cols]
            upper = self.params[cols] + q * bse[cols]
        return np.asarray(lzip(lower, upper))

def lzip(*args, **kwargs):
    return list(zip(*args, **kwargs))



import pysal
import os.path
import pysal.core.FileIO as FileIO
from pysal.weights import W
from warnings import warn

__author__ = "Myunghwa Hwang <mhwang4@gmail.com>"
__all__ = ["GeoBUGSTextIO"]


class GeoBUGSTextIO(FileIO.FileIO):
    """
    Opens, reads, and writes weights file objects in the text format
    used in GeoBUGS. GeoBUGS generates a spatial weights matrix
    as an R object and writes it out as an ASCII text representation of
    the R object.

    An exemplary GeoBUGS text file is as follows.
    list([CARD],[ADJ],[WGT],[SUMNUMNEIGH])
    where [CARD] and [ADJ] are required but the others are optional.
    PySAL assumes [CARD] and [ADJ] always exist in an input text file.
    It can read a GeoBUGS text file, even when its content is not written
    in the order of [CARD], [ADJ], [WGT], and [SUMNUMNEIGH].
    It always writes all of [CARD], [ADJ], [WGT], and [SUMNUMNEIGH].
    PySAL does not apply text wrapping during file writing.

    In the above example,

    [CARD]:
        num=c([a list of comma-splitted neighbor cardinalities])

    [ADJ]:
        adj=c([a list of comma-splitted neighbor IDs])
        if caridnality is zero, neighbor IDs are skipped.
        The ordering of observations is the same in both [CARD] and
        [ADJ].
        Neighbor IDs are record numbers starting from one.

    [WGT]:
        weights=c([a list of comma-splitted weights])
        The restrictions for [ADJ] also apply to [WGT].

    [SUMNUMNEIGH]: 
        sumNumNeigh=[The total number of neighbor pairs]
        the total number of neighbor pairs  is an integer
        value and the same as the sum of neighbor cardinalities.

    Notes
    -----
    For the files generated from R spdep nb2WB and dput function,
    it is assumed that the value for the control parameter of dput function
    is NULL. Please refer to R spdep nb2WB function help file.

    References
    ----------
    Thomas, A., Best, N., Lunn, D., Arnold, R., and Spiegelhalter, D.

    (2004) GeoBUGS User Manual.

    R spdep nb2WB function help file.

    """

    FORMATS = ['geobugs_text']
    MODES = ['r', 'w']

    def __init__(self, *args, **kwargs):
        args = args[:2]
        FileIO.FileIO.__init__(self, *args, **kwargs)
        self.file = open(self.dataPath, self.mode)

    def read(self, n=-1):
        """
        Reads GeoBUGS text file

        Returns
        -------
        a pysal.weights.weights.W object

        Examples
        --------

        Type 'dir(w)' at the interpreter to see what methods are supported.
        Open a GeoBUGS text file and read it into a pysal weights object

        >>> w = pysal.open(pysal.examples.get_path('geobugs_scot'),'r','geobugs_text').read()
        WARNING: there are 3 disconnected observations
        Island ids:  [6, 8, 11]

        Get the number of observations from the header

        >>> w.n
        56

        Get the mean number of neighbors

        >>> w.mean_neighbors
        4.1785714285714288

        Get neighbor distances for a single observation

        >>> w[1]
        {9: 1.0, 19: 1.0, 5: 1.0}

        """
        self._complain_ifclosed(self.closed)
        return self._read()

    def seek(self, pos):
        if pos == 0:
            self.file.seek(0)
            self.pos = 0

    def _read(self):
        if self.pos > 0:
            raise StopIteration

        fbody = self.file.read()
        body_structure = {}
        for i in ['num', 'adj', 'weights', 'sumNumNeigh']:
            i_loc = fbody.find(i)
            if i_loc != -1:
                body_structure[i] = (i_loc, i)
        body_sequence = sorted(body_structure.values())
        body_sequence.append((-1, 'eof'))

        for i in range(len(body_sequence) - 1):
            part, next_part = body_sequence[i], body_sequence[i + 1]
            start, end = part[0], next_part[0]
            part_text = fbody[start:end]

            part_length, start, end = len(part_text), 0, -1
            for c in xrange(part_length):
                if part_text[c].isdigit():
                    start = c
                    break

            for c in xrange(part_length - 1, 0, -1):
                if part_text[c].isdigit():
                    end = c + 1
                    break
            part_text = part_text[start: end]
            part_text = part_text.replace('\n', '')
            value_type = int
            if part[1] == 'weights':
                value_type = float
            body_structure[part[1]] = [value_type(v)
                                       for v in part_text.split(',')]

        cardinalities = body_structure['num']
        adjacency = body_structure['adj']
        raw_weights = [1.0] * int(sum(cardinalities))
        if 'weights' in body_structure and isinstance(body_structure['weights'], list):
            raw_weights = body_structure['weights']

        no_obs = len(cardinalities)
        neighbors = {}
        weights = {}
        pos = 0
        for i in xrange(no_obs):
            neighbors[i + 1] = []
            weights[i + 1] = []
            no_nghs = cardinalities[i]
            if no_nghs > 0:
                neighbors[i + 1] = adjacency[pos: pos + no_nghs]
                weights[i + 1] = raw_weights[pos: pos + no_nghs]
            pos += no_nghs

        self.pos += 1
        return W(neighbors, weights)

    def write(self, obj):
        """
        Writes a weights object to the opened text file.

        Parameters
        ----------
        .write(weightsObject)
        accepts a weights object

        Returns
        ------

        a GeoBUGS text file

        Examples
        --------

        >>> import tempfile, pysal, os
        >>> testfile = pysal.open(pysal.examples.get_path('geobugs_scot'),'r','geobugs_text')
        >>> w = testfile.read()
        WARNING: there are 3 disconnected observations
        Island ids:  [6, 8, 11]

        Create a temporary file for this example

        >>> f = tempfile.NamedTemporaryFile(suffix='')

        Reassign to new var

        >>> fname = f.name

        Close the temporary named file

        >>> f.close()

        Open the new file in write mode

        >>> o = pysal.open(fname,'w','geobugs_text')

        Write the Weights object into the open file

        >>> o.write(w)
        >>> o.close()

        Read in the newly created text file

        >>> wnew =  pysal.open(fname,'r','geobugs_text').read()
        WARNING: there are 3 disconnected observations
        Island ids:  [6, 8, 11]

        Compare values from old to new

        >>> wnew.pct_nonzero == w.pct_nonzero
        True

        Clean up temporary file created for this example

        >>> os.remove(fname)

        """
        self._complain_ifclosed(self.closed)
        if issubclass(type(obj), W):

            cardinalities, neighbors, weights = [], [], []
            for i in obj.id_order:
                cardinalities.append(obj.cardinalities[i])
                neighbors.extend(obj.neighbors[i])
                weights.extend(obj.weights[i])

            self.file.write('list(')
            self.file.write('num=c(%s),' % ','.join(map(str, cardinalities)))
            self.file.write('adj=c(%s),' % ','.join(map(str, neighbors)))
            self.file.write('sumNumNeigh=%i)' % sum(cardinalities))
            self.pos += 1

        else:
            raise TypeError("Expected a pysal weights object, got: %s" % (
                type(obj)))

    def close(self):
        self.file.close()
        FileIO.FileIO.close(self)




import pysal
from pysal.cg.segmentLocator import Polyline_Shapefile_SegmentLocator
import networkx

EUCLIDEAN_DISTANCE = "Euclidean"
ARC_DISTANCE = "Arc"

class SpatialNetwork(object):
    """
    SpatialNetwork -- Represents a Spatial Network. 

    A Spatial Network in PySAL is a graph who's nodes and edges are represented by geographic features 
    such as Points for nodes and Lines for edges.
    An example of a spatial network is a road network.

    Arguments
    ---------
    shapefile -- Shapefile contains the geographic represention of the network.
                 The shapefile must be a polyline type and the associated DBF MUST contain the following fields:
                    FNODE -- source node -- ID of the source node, the first vertex in the polyline feature.
                    TNODE -- destination node -- ID of the destination node, the last vertex in the polyline feature.
                    ONEWAY -- bool -- If True, the edge will be marked oneway starting at FNODE and ending at TNODE
    distance_metric -- EUCLIDEAN_DISTANCE or ARC_DISTANCE
    
    """
    def __init__(self,shapefile,distance_metric=EUCLIDEAN_DISTANCE):
        if issubclass(type(shapefile),basestring): #Path
            self.shp = shp = pysal.open(shapefile,'r')
        else:
            raise TypeError,"Expecting a string, shapefile should the path to shapefile"
        if shp.type != pysal.cg.shapes.Chain:
            raise ValueError,"Shapefile must contain polyline features"
        self.dbf = dbf = pysal.open(shapefile[:-4]+'.dbf','r')
        header = dbf.header
        if (('FNODE' not in header) or ('TNODE' not in header) or ('ONEWAY' not in header)):
            raise ValueError,"DBF must contain: FNODE,TNODE,ONEWAY"
        
        oneway = [{'F':False,'T':True}[x] for x in dbf.by_col('ONEWAY')]
        fnode = dbf.by_col('FNODE')
        tnode = dbf.by_col('TNODE')
        if distance_metric == EUCLIDEAN_DISTANCE:
            lengths = [x.len for x in shp]
        elif distance_metric == ARC_DISTANCE:
            lengths = [x.arclen for x in shp]
        else:
            raise ValueError,"distance_metric must be either EUCLIDEAN_DISTANCE or ARC_DISTANCE"
        self.lengths = lengths
        if any(oneway):
            G = networkx.MultiDiGraph()
            #def isoneway(x):
            #    return x[-1]
            #def isnotoneway(x):
            #    if(x[-1]):
            #        return False
            #    return True
            #    
            #A = filter(isoneway,zip(fnode,tnode,oneway))
            #B = filter(isnotoneway,zip(fnode,tnode,oneway))
            #C = filter(isnotoneway,zip(tnode,fnode,oneway))
            #G.add_edges_from(A)
            #G.add_edges_from(B)
            #G.add_edges_from(C)
        else:
            G = networkx.MultiGraph()
        #zip(fnode,tnode))
        self.G = G
        shp.seek(0)
        self._locator = Polyline_Shapefile_SegmentLocator(shp)
    def snap(self,pt):
        i,p,j = self._locator.nearest(pt) #shpID,partID,segmentID
        segment = self.shp[i].segments[p][j] #grab segment
        d,pct = pysal.cg.get_segment_point_dist(segment,pt) #find pct along segment
        x0,x1 = segment.p1[0],segment.p2[0]
        x2 = x0 + (x1-x0)*pct # find x location of snap
        y2 = segment.line.y(x2) # find y location of snap

        #dbf = self.dbf
        #rec = dict(zip(dbf.header,dbf[i][0]))
        #edge = (rec['FNODE'],rec['TNODE'])
        #TODO: Calculate location along edge and distance to edge"
        #return edge
        return x2,y2
        
    
if __name__=='__main__':
    import random
    net = SpatialNetwork('beth_network.shp',ARC_DISTANCE)

    n = 1000
    minX,minY,maxX,maxY = net.shp.bbox
    xRange = maxX-minX
    yRange = maxY-minY
    qpts = [(random.random(), random.random()) for i in xrange(n)]
    qpts = [pysal.cg.Point((minX+(xRange*x),minY+(yRange*y))) for x,y in qpts]
    o = pysal.open('random_qpts.shp','w')
    for p in qpts:
        o.write(p)
    o.close()
    o = pysal.open('random_qpts_snapped.shp','w')
    for qpt in qpts:
        spt = net.snap(qpt)
        o.write(pysal.cg.Chain([qpt,spt]))
    o.close()
    






"""
color handling for mapping and geovisualization
"""

from warnings import warn
try:
    import palettable as pltt
    from palettable.colorbrewer import qualitative, sequential, diverging
except:
    warn('palettable  not installed. Functionality '
          'related to it will not work')

def get_color_map(palette=None, name='BuGn', cmtype='sequential', k=5,
                  color_encoding='hex'):
    """
    Get a brewer colormap from `palettable`

    Arguments
    ---------

    palette         : `palettable` palette
                      Palettable object for a given palette
    name            : string
                      colormap name

    cmtype          : string
                      colormap scale type  [sequential, diverging, qualitative]

    k               : int
                      number of classes

    color_encoding  : string
                      encoding of colors [hexc, rgb, mpl, mpl_colormap]
                      * hex: list of hex strings
                      * rgb: list of RGB 0-255 triplets
                      * mpl: list of RGB 0-1 triplets as used by matplotlib
                      * mpl_colormap: matplotlib color map

    Returns
    -------

    colors:  color map in the specified color_encoding

    """
    encs = {'hex': 'hex_colors',
            'rgb': 'colors',
            'mpl': 'mpl_colors',
            'mpl_colormap': 'mpl_colormap'}
    if not palette:
        cmtype = pltt2type[name.lower()]
        if name[-2:] == '_r':
            palette = pltt.colorbrewer.get_map(name[:-2], cmtype,
                    k, reverse=True)
        else:
            palette = pltt.colorbrewer.get_map(name, cmtype, k)
    colors = getattr(palette, encs[color_encoding.lower()])
    return colors

def _build_pltt2type():
    types = ['sequential', 'diverging', 'qualitative']
    pltt2type = {}
    for t in types:
        pals = list(set([
            p.split('_')[0] for p in dir(getattr(pltt.colorbrewer, t))
                            if p[0]!='_'
                            ]))
        pals = pals + [p+'_r' for p in pals]
        for p in pals:
            pltt2type[p.lower()] = t
    return pltt2type

pltt2type = _build_pltt2type()

def get_maps_by_type(data_type):
    names = [name for name in dir(data_type) if not name.startswith('_')]
    return names




import ops

def set_preference(pkg):
    """
    Set which provider of geospatial operations should be preferred. 

    Arguments
    ---------
    pkg     :   string
                string specifying whether to prefer computing properties using
                shapely, or to use attribute access on the polygons themselves.
    Returns
    --------
    None, sets configuration option directly
    """
    if pkg.lower() == 'shapely':
        ops.atomic.__dict__.update({k:v for k,v in
                                    ops.atomic._s.__dict__.items() 
                                    if not k.startswith('_')})
        ops.atomic._preferred = ops.atomic._s
    elif pkg.lower().startswith('att'):
        ops.atomic.__dict__.update({k:v for k,v in
                                    ops.atomic._a.__dict__.items() 
                                    if not k.startswith('_')})
        ops.atomic._preferred = ops.atomic._a
    else:
        raise Exception('Provider not recognized.')

def get_provider(fn=None):
    """
    Discover the provider of a given function

    Arguments
    ---------
    fn      :   Callable or None
                either a function whose provider needs to be discovered, or None

    Returns
    -------
    the module where fn is defined, or the current preferred provider. 

    Example
    -------

    >>> from pysal.contrib.geotable import ops
    >>> from pysal.contrib.config import get_provider, set_config
    >>> get_provider(ops.atomic.area)
        <module 'pysal.contrib.geotable.ops._accessors' from 
         pysal/contrib/geotable/ops/_accessors.pyc'>
    >>> set_preference('shapely')
    >>> get_provider(ops.atomic.area)
        <module 'pysal.contrib.geotable.ops._shapely' from
        'pysal/contrib/geotable/ops/_shapely.pyc'>
    """
    if fn is None:
        return ops.atomic._preferred
    try:
        fprovenance = fn.func.func_name
    except AttributeError:
        fprovenance = fn.func_name
    if fprovenance == 'get_attr':
        return ops._accessors
    else:
        return ops._shapely



import unittest
import pysal
from pysal.esda import smoothing as sm
from pysal import knnW_from_array
import numpy as np
from pysal.common import RTOL, ATOL

from pysal.common import pandas

PANDAS_EXTINCT = pandas is None

class TestFlatten(unittest.TestCase):
    def setUp(self):
        self.input = [[1, 2], [3, 3, 4], [5, 6]]

    def test_flatten(self):
        out1 = sm.flatten(self.input)
        out2 = sm.flatten(self.input, unique=False)
        self.assertEquals(out1, [1, 2, 3, 4, 5, 6])
        self.assertEquals(out2, [1, 2, 3, 3, 4, 5, 6])


class TestWMean(unittest.TestCase):
    def setUp(self):
        self.d = np.array([5, 4, 3, 1, 2])
        self.w1 = np.array([10, 22, 9, 2, 5])
        self.w2 = np.array([10, 14, 17, 2, 5])

    def test_weighted_median(self):
        out1 = sm.weighted_median(self.d, self.w1)
        out2 = sm.weighted_median(self.d, self.w2)
        self.assertEquals(out1, 4)
        self.assertEquals(out2, 3.5)


class TestAgeStd(unittest.TestCase):
    def setUp(self):
        self.e = np.array([30, 25, 25, 15, 33, 21, 30, 20])
        self.b = np.array([1000, 1000, 1100, 900, 1000, 900, 1100, 900])
        self.s_e = np.array([100, 45, 120, 100, 50, 30, 200, 80])
        self.s_b = s = np.array([1000, 900, 1000, 900, 1000, 900, 1000, 900])
        self.n = 2

    def test_crude_age_standardization(self):
        crude = sm.crude_age_standardization(self.e, self.b, self.n).round(8)
        crude_exp = np.array([0.02375000, 0.02666667])
        self.assertEquals(list(crude), list(crude_exp))

    def test_direct_age_standardization(self):
        direct = np.array(sm.direct_age_standardization(
            self.e, self.b, self.s_b, self.n)).round(8)
        direct_exp = np.array([[0.02374402, 0.01920491,
                                0.02904848], [0.02665072, 0.02177143, 0.03230508]])
        self.assertEquals(list(direct.flatten()), list(direct_exp.flatten()))

    def test_indirect_age_standardization(self):
        indirect = np.array(sm.indirect_age_standardization(
            self.e, self.b, self.s_e, self.s_b, self.n)).round(8)
        indirect_exp = np.array([[0.02372382, 0.01940230,
                                  0.02900789], [0.02610803, .02154304, 0.03164035]])
        self.assertEquals(
            list(indirect.flatten()), list(indirect_exp.flatten()))


class TestSRate(unittest.TestCase):
    def setUp(self):
        sids = pysal.open(pysal.examples.get_path('sids2.dbf'), 'r')
        self.w = pysal.open(pysal.examples.get_path('sids2.gal'), 'r').read()
        self.b, self.e = np.array(sids[:, 8]), np.array(sids[:, 9])
        self.er = [0.453433, 0.000000, 0.775871, 0.973810, 3.133190]
        self.eb = [0.0016973, 0.0017054, 0.0017731, 0.0020129, 0.0035349]
        self.sr = [0.0009922, 0.0012639, 0.0009740, 0.0007605, 0.0050154]
        self.smr = [0.00083622, 0.00109402, 0.00081567, 0.0, 0.0048209]
        self.smr_w = [0.00127146, 0.00127146, 0.0008433, 0.0, 0.0049889]
        self.smr2 = [0.00091659, 0.00087641, 0.00091073, 0.0, 0.00467633]
        self.s_ebr10 = np.array([4.01485749e-05, 3.62437513e-05,
                            4.93034844e-05, 5.09387329e-05, 3.72735210e-05,
                            3.69333797e-05, 5.40245456e-05, 2.99806055e-05,
                            3.73034109e-05, 3.47270722e-05]).reshape(-1,1)

        self.stl = pysal.open(pysal.examples.get_path('stl_hom.csv'), 'r')
        self.stl_e, self.stl_b = np.array(self.stl[:, 10]), np.array(self.stl[:, 13])
        self.stl_w = pysal.open(pysal.examples.get_path('stl.gal'), 'r').read()
        if not self.stl_w.id_order_set:
            self.stl_w.id_order = range(1, len(self.stl) + 1)

        if not PANDAS_EXTINCT:
            self.df = pysal.open(pysal.examples.get_path('sids2.dbf')).to_df()
            self.ename = 'SID74'
            self.bname = 'BIR74'
            self.stl_df = pysal.open(pysal.examples.get_path('stl_hom.csv')).to_df()
            self.stl_ename = 'HC7984'
            self.stl_bname = 'PO7984'

    def test_Excess_Risk(self):
        out_er = sm.Excess_Risk(self.e, self.b).r
        np.testing.assert_allclose(out_er[:5].flatten(), self.er,
                                   rtol=RTOL, atol=ATOL)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Excess_Risk_tabular(self):
        out_er = sm.Excess_Risk(self.df[self.ename], self.df[self.bname]).r
        np.testing.assert_allclose(out_er[:5].flatten(), self.er,
                                   rtol=RTOL, atol=ATOL)
        self.assertIsInstance(out_er, np.ndarray)
        out_er = sm.Excess_Risk.by_col(self.df, self.ename, self.bname)
        outcol = '{}-{}_excess_risk'.format(self.ename, self.bname)
        outcol = out_er[outcol]
        np.testing.assert_allclose(outcol[:5], self.er,
                                   rtol=RTOL, atol=ATOL)
        self.assertIsInstance(outcol.values, np.ndarray)

    def test_Empirical_Bayes(self):
        out_eb = sm.Empirical_Bayes(self.e, self.b).r
        np.testing.assert_allclose(out_eb[:5].flatten(), self.eb,
                                   rtol=RTOL, atol=ATOL)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Empirical_Bayes_tabular(self):
        out_eb = sm.Empirical_Bayes(self.df[self.ename], self.df[self.bname]).r
        np.testing.assert_allclose(out_eb[:5].flatten(), self.eb,
                                   rtol=RTOL, atol=ATOL)
        self.assertIsInstance(out_eb, np.ndarray)

        out_eb = sm.Empirical_Bayes.by_col(self.df, self.ename, self.bname)
        outcol = '{}-{}_empirical_bayes'.format(self.ename, self.bname)
        outcol = out_eb[outcol]
        np.testing.assert_allclose(outcol[:5], self.eb,
                                   rtol=RTOL, atol=ATOL)
        self.assertIsInstance(outcol.values, np.ndarray)

    def test_Spatial_Empirical_Bayes(self):
        s_eb = sm.Spatial_Empirical_Bayes(self.stl_e, self.stl_b, self.stl_w)
        np.testing.assert_allclose(self.s_ebr10, s_eb.r[:10],
                                   rtol=RTOL, atol=ATOL)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Spatial_Empirical_Bayes_tabular(self):
        s_eb = sm.Spatial_Empirical_Bayes(self.stl_df[self.stl_ename],
                                          self.stl_df[self.stl_bname],
                                          self.stl_w).r
        self.assertIsInstance(s_eb, np.ndarray)
        np.testing.assert_allclose(self.s_ebr10, s_eb[:10])

        s_eb = sm.Spatial_Empirical_Bayes.by_col(self.stl_df,
                                                 self.stl_ename,
                                                 self.stl_bname,
                                                 self.stl_w)
        outcol = '{}-{}_spatial_empirical_bayes'.format(self.stl_ename, self.stl_bname)
        r = s_eb[outcol].values
        self.assertIsInstance(r, np.ndarray)
        np.testing.assert_allclose(self.s_ebr10, r[:10].reshape(-1,1))

    def test_Spatial_Rate(self):
        out_sr = sm.Spatial_Rate(self.e, self.b, self.w).r
        np.testing.assert_allclose(out_sr[:5].flatten(), self.sr,
                                   rtol=RTOL, atol=ATOL)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Spatial_Rate_tabular(self):
        out_sr = sm.Spatial_Rate(self.df[self.ename], self.df[self.bname], self.w).r
        np.testing.assert_allclose(out_sr[:5].flatten(), self.sr,
                                   rtol=RTOL, atol=ATOL)
        self.assertIsInstance(out_sr, np.ndarray)

        out_sr = sm.Spatial_Rate.by_col(self.df, self.ename, self.bname,w=self.w)
        outcol = '{}-{}_spatial_rate'.format(self.ename, self.bname)
        r = out_sr[outcol].values
        self.assertIsInstance(r, np.ndarray)
        np.testing.assert_allclose(r[:5], self.sr, rtol=RTOL, atol=ATOL)

    def test_Spatial_Median_Rate(self):
        out_smr = sm.Spatial_Median_Rate(self.e, self.b, self.w).r
        out_smr_w = sm.Spatial_Median_Rate(self.e, self.b, self.w, aw=self.b).r
        out_smr2 = sm.Spatial_Median_Rate(self.e, self.b, self.w, iteration=2).r
        np.testing.assert_allclose(out_smr[:5].flatten(), self.smr,
                                   atol=ATOL, rtol=RTOL)
        np.testing.assert_allclose(out_smr_w[:5].flatten(), self.smr_w,
                                   atol=ATOL, rtol=RTOL)
        np.testing.assert_allclose(out_smr2[:5].flatten(), self.smr2,
                                   atol=ATOL, rtol=RTOL)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Spatial_Median_Rate_tabular(self):
        out_smr = sm.Spatial_Median_Rate(self.df[self.ename],
                                         self.df[self.bname],
                                         self.w).r
        out_smr_w = sm.Spatial_Median_Rate(self.df[self.ename],
                                           self.df[self.bname],
                                           self.w,
                                           aw = self.df[self.bname]).r
        out_smr2 = sm.Spatial_Median_Rate(self.df[self.ename],
                                          self.df[self.bname],
                                          self.w,
                                          iteration=2).r

        self.assertIsInstance(out_smr, np.ndarray)
        self.assertIsInstance(out_smr_w, np.ndarray)
        self.assertIsInstance(out_smr2, np.ndarray)

        np.testing.assert_allclose(out_smr[:5].flatten(), self.smr,
                                   atol=ATOL, rtol=RTOL)
        np.testing.assert_allclose(out_smr_w[:5].flatten(), self.smr_w,
                                   atol=ATOL, rtol=RTOL)
        np.testing.assert_allclose(out_smr2[:5].flatten(), self.smr2,
                                   atol=ATOL, rtol=RTOL)

        out_smr = sm.Spatial_Median_Rate.by_col(self.df, self.ename,
                                                self.bname, self.w)
        out_smr_w = sm.Spatial_Median_Rate.by_col(self.df, self.ename,
                                                  self.bname, self.w,
                                                  aw = self.df[self.bname])
        out_smr2 = sm.Spatial_Median_Rate.by_col(self.df, self.ename,
                                                 self.bname, self.w,
                                                 iteration=2)
        outcol = '{}-{}_spatial_median_rate'.format(self.ename, self.bname)

        np.testing.assert_allclose(out_smr[outcol].values[:5], self.smr,
                                   rtol=RTOL, atol=ATOL)
        np.testing.assert_allclose(out_smr_w[outcol].values[:5], self.smr_w,
                                   rtol=RTOL, atol=ATOL)
        np.testing.assert_allclose(out_smr2[outcol].values[:5], self.smr2,
                                   rtol=RTOL, atol=ATOL)

        @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
        def test_Spatial_Smoother_multicol(self):
            """
            test that specifying multiple columns works correctly. Since the
            function is shared over all spatial smoothers, we can only test one.
            """
            enames = [self.ename, 'SID79']
            bnames = [self.bname, 'BIR79']
            out_df = sm.Spatial_Median_Rate.by_col(self.df, enames, bnames, self.w)
            outcols = ['{}-{}_spatial_median_rate'.format(e,b)
                       for e,b in zip(enames, bnames)]
            smr79 = np.array([0.00122129, 0.00176924, 0.00176924,
                              0.00240964, 0.00272035])
            answers = [self.smr, smr79]
            for col, answer in zip(outcols, answer):
                self.assertIn(out_df.columns, col)
                np.testing.assert_allclose(out_df[col].values[:5], answer,
                                           rtol=RTOL, atol=ATOL)

        @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
        def test_Smoother_multicol(self):
            """
            test that non-spatial smoothers work with multicolumn queries
            """
            enames = [self.ename, 'SID79']
            bnames = [self.bname, 'BIR79']
            out_df = sm.Excess_Risk.by_col(self.df, enames, bnames)
            outcols = ['{}-{}_excess_risk'.format(e,b)
                       for e,b in zip(enames, bnames)]
            er79 = np.array([0.000000, 2.796607, 0.8383863,
                              1.217479, 0.943811])
            answers = [self.er, er79]
            for col, answer in zip(outcols, answer):
                self.assertIn(out_df.columns, col)
                np.testing.assert_allclose(out_df[col].values[:5], answer,
                                           rtol=RTOL, atol=ATOL)

class TestHB(unittest.TestCase):
    def setUp(self):
        sids = pysal.open(pysal.examples.get_path('sids2.shp'), 'r')
        self.sids = sids
        self.d = np.array([i.centroid for i in sids])
        self.w = knnW_from_array(self.d, k=5)
        if not self.w.id_order_set:
            self.w.id_order = self.w.id_order
        sids_db = pysal.open(pysal.examples.get_path('sids2.dbf'), 'r')
        self.b, self.e = np.array(sids_db[:, 8]), np.array(sids_db[:, 9])
        self.sids_hb_rr5 = np.array([0.00075586, 0.,
                                     0.0008285, 0.0018315, 0.00498891])
        self.sids_hb_r2r5 = np.array([0.0008285, 0.00084331,
                                      0.00086896, 0.0018315, 0.00498891])
        self.sids_hb_r3r5 = np.array([0.00091659, 0.,
                                      0.00156838, 0.0018315, 0.00498891])
        if not PANDAS_EXTINCT:
            self.df = sids_db.to_df()
            self.ename = 'SID74'
            self.bname = 'BIR74'
            self.enames = [self.ename, 'SID79']
            self.bnames = [self.bname, 'BIR79']
            self.sids79r = np.array([.000563, .001659, .001879,
                                     .002410, .002720])
    @unittest.skip("Depreication")
    def test_Headbanging_Triples(self):
        ht = sm.Headbanging_Triples(self.d, self.w)
        self.assertEquals(len(ht.triples), len(self.d))
        ht2 = sm.Headbanging_Triples(self.d, self.w, edgecor=True)
        self.assertTrue(hasattr(ht2, 'extra'))
        self.assertEquals(len(ht2.triples), len(self.d))
        htr = sm.Headbanging_Median_Rate(self.e, self.b, ht2, iteration=5)
        self.assertEquals(len(htr.r), len(self.e))
        for i in htr.r:
            self.assertTrue(i is not None)

    @unittest.skip("Deprication")
    def test_Headbanging_Median_Rate(self):
        s_ht = sm.Headbanging_Triples(self.d, self.w, k=5)
        sids_hb_r = sm.Headbanging_Median_Rate(self.e, self.b, s_ht)
        np.testing.assert_array_almost_equal(self.sids_hb_rr5, sids_hb_r.r[:5])
        sids_hb_r2 = sm.Headbanging_Median_Rate(self.e, self.b, s_ht, iteration=5)
        np.testing.assert_array_almost_equal(self.sids_hb_r2r5, sids_hb_r2.r[:5])
        sids_hb_r3 = sm.Headbanging_Median_Rate(self.e, self.b, s_ht, aw=self.b)
        np.testing.assert_array_almost_equal(self.sids_hb_r3r5, sids_hb_r3.r[:5])

    #@unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    @unittest.skip("Deprication")
    def test_Headbanging_Median_Rate_tabular(self):

        # test that dataframe columns are treated correctly
        s_ht = sm.Headbanging_Triples(self.d, self.w, k=5)
        sids_hb_r = sm.Headbanging_Median_Rate(self.df[self.ename],
                                               self.df[self.bname], s_ht)
        self.assertIsInstance(sids_hb_r.r, np.ndarray)
        np.testing.assert_array_almost_equal(self.sids_hb_rr5, sids_hb_r.r[:5])

        sids_hb_r2 = sm.Headbanging_Median_Rate(self.df[self.ename],
                                                self.df[self.bname], s_ht,
                                                iteration=5)
        self.assertIsInstance(sids_hb_r2.r, np.ndarray)
        np.testing.assert_array_almost_equal(self.sids_hb_r2r5, sids_hb_r2.r[:5])

        sids_hb_r3 = sm.Headbanging_Median_Rate(self.df[self.ename],
                                                self.df[self.bname], s_ht,
                                                aw=self.df[self.bname])
        self.assertIsInstance(sids_hb_r3.r, np.ndarray)
        np.testing.assert_array_almost_equal(self.sids_hb_r3r5, sids_hb_r3.r[:5])

        #test that the by col on multiple names works correctly
        sids_hr_df = sm.Headbanging_Median_Rate.by_col(self.df, self.enames,
                                                       self.bnames, w=self.w)
        outcols = ['{}-{}_headbanging_median_rate'.format(e,b) for e,b in
                   zip(self.enames, self.bnames)]
        for col, answer in zip(outcols, [self.sids_hb_rr5, self.sids79r]):
            this_col = sids_hr_df[col].values
            self.assertIsInstance(this_col, np.ndarray)
            np.testing.assert_allclose(sids_hr_df[col][:5], answer,
                                       rtol=RTOL, atol=ATOL*10)

class TestKernel_AgeAdj_SM(unittest.TestCase):
    def setUp(self):
        self.e = np.array([10, 1, 3, 4, 2, 5])
        self.b = np.array([100, 15, 20, 20, 80, 90])
        self.e1 = np.array([10, 8, 1, 4, 3, 5, 4, 3, 2, 1, 5, 3])
        self.b1 = np.array([100, 90, 15, 30, 25, 20, 30, 20, 80, 80, 90, 60])
        self.s = np.array([98, 88, 15, 29, 20, 23, 33, 25, 76, 80, 89, 66])
        self.points = [(
            10, 10), (20, 10), (40, 10), (15, 20), (30, 20), (30, 30)]
        self.kw = pysal.weights.Kernel(self.points)
        self.points1 = np.array(self.points) + 5
        self.points1 = np.vstack((self.points1, self.points))
        self.kw1 = pysal.weights.Kernel(self.points1)
        if not self.kw.id_order_set:
            self.kw.id_order = range(0, len(self.points))
        if not PANDAS_EXTINCT:
            import pandas as pd
            dfa = np.array([self.e, self.b]).T
            dfb = np.array([self.e1, self.b1, self.s]).T
            self.dfa = pd.DataFrame(dfa, columns=['e','b'])
            self.dfb = pd.DataFrame(dfb, columns=['e', 'b', 's'])

        #answers
        self.kernel_exp = [0.10543301, 0.0858573, 0.08256196, 0.09884584,
                           0.04756872, 0.04845298]
        self.ageadj_exp = [0.10519625, 0.08494318, 0.06440072, 0.06898604,
                           0.06952076, 0.05020968]
        self.disk_exp = [0.12222222000000001, 0.10833333, 0.08055556,
                         0.08944444, 0.09944444, 0.09351852]
        self.sf_exp = np.array([ 0.111111,  0.111111,  0.085106,  0.076923])

    def test_Kernel_Smoother(self):
        kr = sm.Kernel_Smoother(self.e, self.b, self.kw)
        np.testing.assert_allclose(kr.r.flatten(), self.kernel_exp)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Kernel_Smoother_tabular(self):
        dfa, dfb = self.dfa, self.dfb
        kr = sm.Kernel_Smoother(dfa['e'], dfa['b'], self.kw)
        np.testing.assert_allclose(kr.r.flatten(), kernel_exp)

        kr = sm.Kernel_Smoother.by_col(dfa, 'e', 'b', w=self.kw)
        colname = 'e_b_kernel_smoother'
        np.testing.assert_allclose(kr[colname].values, kernel_exp)

        kr = sm.Kernel_Smoother.by_col(dfb, ['e', 's'], 'b', w=self.kw)
        outcols = ['{}-b_kernel_smoother'.format(l) for l in ['e','s']]

        exp_eb = np.array([ 0.08276363,  0.08096262,  0.03636364,  0.0704302 ,
                            0.07996067,  0.1287226 ,  0.09831286,  0.0952105 ,
                            0.02857143,  0.06671039,  0.07129231,  0.08078792])
        exp_sb = np.array([ 1.00575463,  0.99597005,  0.96363636,  0.99440132,
                            0.98468399,  1.07912333,  1.03376267,  1.02759815,
                            0.95428572,  0.99716186,  0.98277235,  1.03906155])
        for name, answer in zip(outcols, [exp_eb, exp_sb]):
            np.testing.assert_allclose(kr[name].values, answer, rtol=RTOL, atol=ATOL)

    def test_Age_Adjusted_Smoother(self):
        ar = sm.Age_Adjusted_Smoother(self.e1, self.b1, self.kw, self.s)
        np.testing.assert_allclose(ar.r, self.ageadj_exp)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Age_Adjusted_Smoother_tabular(self):
        dfb = self.dfb
        kr = sm.Age_Adjusted_Smoother(dfb.e, dfb.b, s=dfb.s, w=self.kw)
        self.assertIsInstance(kr.r, np.ndarray)
        np.testing.assert_allclose(kr.r, self.ageadj_exp)

        kr = sm.Age_Adjusted_Smoother.by_col(dfb, 'e', 'b', w=self.kw, s='s')
        answer = np.array([ 0.10519625, 0.08494318, 0.06440072,
                         0.06898604, 0.06952076, 0.05020968])
        colname = 'e-b_age_adjusted_smoother'
        np.testing.assert_allclose(kr[colname].values, answer, rtol=RTOL, atol=ATOL)

    def test_Disk_Smoother(self):
        self.kw.transform = 'b'
        disk = sm.Disk_Smoother(self.e, self.b, self.kw)
        np.testing.assert_allclose(disk.r.flatten(), self.disk_exp)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Disk_Smoother_tabular(self):
        self.kw.transform = 'b'
        dfa = self.dfa
        disk = sm.Disk_Smoother(dfa.e, dfa.b, self.kw).r
        np.testing.assert_allclose(disk.flatten(), self.disk_exp)

        disk = sm.Disk_Smoother.by_col(dfa, 'e', 'b', self.kw)
        col = 'e-b_disk_smoother'
        np.testing.assert_allclose(disk[col].values, self.disk_exp,
                                   rtol=RTOL, atol=ATOL)

    def test_Spatial_Filtering(self):
        points = np.array(self.points)
        bbox = [[0, 0], [45, 45]]
        sf = sm.Spatial_Filtering(bbox, points, self.e, self.b, 2, 2, r=30)
        np.testing.assert_allclose(sf.r, self.sf_exp, rtol=RTOL, atol=ATOL)

    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
    def test_Kernel_Smoother_tabular(self):
        point_array = np.array(self.points)
        bbox = [[0,0] , [45,45]]
        dfa = self.dfa
        sf = sm.Spatial_Filtering(bbox, point_array, dfa.e, dfa.b, 2, 2, r=30)

        np.testing.assert_allclose(sf.r, self.sf_exp, rtol=RTOL, atol=ATOL)

        dfa['geometry'] = self.points
        sf = sm.Spatial_Filtering.by_col(dfa, 'e', 'b', 3, 3, r=30)
        r_answer = np.array([ 0.07692308,  0.07213115,  0.07213115,  0.07692308,
                              0.07692308,  0.07692308,  0.07692308,  0.07692308,
                              0.07692308])
        x_answer = np.array([10.0, 10.0, 10.0, 20.0, 20.0, 20.0, 30.0, 30.0, 30.0])
        y_answer = np.array([10.000000, 16.666667, 23.333333,
                             10.000000, 16.666667, 23.333333,
                             10.000000, 16.666667, 23.333333])
        columns = ['e-b_spatial_filtering_{}'.format(name) for name in ['X', 'Y', 'R']]

        for col, answer in zip(columns, [x_answer, y_answer, r_answer]):
            np.testing.assert_allclose(sf[col].values, answer, rtol=RTOL, atol=ATOL)

class TestUtils(unittest.TestCase):
    def test_sum_by_n(self):
        d = np.array([10, 9, 20, 30])
        w = np.array([0.5, 0.1, 0.3, 0.8])
        n = 2
        exp_sum = np.array([5.9, 30.])
        np.testing.assert_array_almost_equal(exp_sum, sm.sum_by_n(d, w, n))

    def test_standardized_mortality_ratio(self):
        e = np.array([30, 25, 25, 15, 33, 21, 30, 20])
        b = np.array([100, 100, 110, 90, 100, 90, 110, 90])
        s_e = np.array([100, 45, 120, 100, 50, 30, 200, 80])
        s_b = np.array([1000, 900, 1000, 900, 1000, 900, 1000, 900])
        n = 2
        exp_smr = np.array([2.48691099, 2.73684211])
        np.testing.assert_array_almost_equal(exp_smr,
                                             sm.standardized_mortality_ratio(e, b, s_e, s_b, n))

    def test_choynowski(self):
        e = np.array([30, 25, 25, 15, 33, 21, 30, 20])
        b = np.array([100, 100, 110, 90, 100, 90, 110, 90])
        n = 2
        exp_choy = np.array([0.30437751, 0.29367033])
        np.testing.assert_array_almost_equal(exp_choy, sm.choynowski(e, b, n))

    def test_assuncao_rate(self):
        e = np.array([30, 25, 25, 15, 33, 21, 30, 20])
        b = np.array([100, 100, 110, 90, 100, 90, 110, 90])
        exp_assuncao = np.array([1.03843594, -0.04099089, -0.56250375,
            -1.73061861])
        np.testing.assert_array_almost_equal(
            exp_assuncao, sm.assuncao_rate(e, b)[:4])


suite = unittest.TestSuite()
test_classes = [TestFlatten, TestWMean, TestAgeStd, TestSRate, TestHB,
                TestKernel_AgeAdj_SM, TestUtils]
for i in test_classes:
    a = unittest.TestLoader().loadTestsFromTestCase(i)
    suite.addTest(a)

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite)



import pysal.core.FileIO as FileIO
from pysal.core.util import WKTParser
from pysal import cg
import re

__author__ = "Charles R Schmidt <schmidtc@gmail.com>"
__all__ = ['WKTReader']
#####################################################################
## ToDo: Add Well-Known-Binary support...
##       * WKB spec:
##  http://webhelp.esri.com/arcgisserver/9.3/dotNet/index.htm#geodatabases/the_ogc_103951442.htm
##
##
#####################################################################


class WKTReader(FileIO.FileIO):
    """
    Parameters
    ----------
    Reads Well-Known Text
    Returns a list of PySAL Polygon objects

    Examples
    --------
    Read in WKT-formatted file

    >>> import pysal
    >>> f = pysal.open(pysal.examples.get_path('stl_hom.wkt'), 'r')

    Convert wkt to pysal polygons

    >>> polys = f.read()

    Check length

    >>> len(polys)
    78

    Return centroid of polygon at index 1

    >>> polys[1].centroid
    (-91.19578469430738, 39.990883050220845)

    Type dir(polys[1]) at the python interpreter to get a list of supported methods

    """
    MODES = ['r']
    FORMATS = ['wkt']

    def __init__(self, *args, **kwargs):
        FileIO.FileIO.__init__(self, *args, **kwargs)
        self.__idx = {}
        self.__pos = 0
        self.__open()

    def open(self):
        self.__open()

    def __open(self):
        self.dataObj = open(self.dataPath, self.mode)
        self.wkt = WKTParser()

    def _read(self):
        FileIO.FileIO._complain_ifclosed(self.closed)
        if self.__pos not in self.__idx:
            self.__idx[self.__pos] = self.dataObj.tell()
        line = self.dataObj.readline()
        if line:
            shape = self.wkt.fromWKT(line)
            shape.id = self.pos
            self.__pos += 1
            self.pos += 1
            return shape
        else:
            self.seek(0)
            return None

    def seek(self, n):
        FileIO.FileIO.seek(self, n)
        pos = self.pos
        if pos in self.__idx:
            self.dataObj.seek(self.__idx[pos])
            self.__pos = pos
        else:
            while pos not in self.__idx:
                s = self._read()
                if not s:
                    raise IndexError("%d not in range(0,%d)" % (
                        pos, max(self.__idx.keys())))
            self.pos = pos
            self.__pos = pos
            self.dataObj.seek(self.__idx[pos])

    def close(self):
        self.dataObj.close()
        FileIO.FileIO.close(self)




"""
PySAL ShapeFile Reader and Writer based on pure python shapefile module.

"""

__author__ = "Charles R Schmidt <schmidtc@gmail.com>"
__credits__ = "Copyright (c) 2009 Charles R. Schmidt"
__all__ = ['PurePyShpWrapper']

#import pysal
import pysal.core.FileIO  # as FileIO
from pysal.core.util import shp_file
import pysal.cg as cg
from warnings import warn
import unittest

STRING_TO_TYPE = {'POLYGON': cg.Polygon, 'POINT': cg.Point, 'POINTM':
                  cg.Point, 'POINTZ': cg.Point, 'ARC': cg.Chain, 'POLYGONZ': cg.Polygon}
TYPE_TO_STRING = {cg.Polygon: 'POLYGON', cg.Point: 'POINT', cg.Chain:
                  'ARC'}  # build the reverse map
#for key,value in STRING_TO_TYPE.iteritems():
#    TYPE_TO_STRING[value] = key


class PurePyShpWrapper(pysal.core.FileIO.FileIO):
    """
    FileIO handler for ESRI ShapeFiles.

    Notes
    -----
    This class wraps _pyShpIO's shp_file class with the PySAL FileIO API.
    shp_file can be used without PySAL.

    Attributes
    ----------

    Formats     : list
                  A list of support file extensions
    Modes       : list
                  A list of support file modes

    Examples
    --------

    >>> import tempfile
    >>> f = tempfile.NamedTemporaryFile(suffix='.shp'); fname = f.name; f.close()
    >>> import pysal
    >>> i = pysal.open(pysal.examples.get_path('10740.shp'),'r')
    >>> o = pysal.open(fname,'w')
    >>> for shp in i:
    ...     o.write(shp)
    >>> o.close()
    >>> open(pysal.examples.get_path('10740.shp'),'rb').read() == open(fname,'rb').read()
    True
    >>> open(pysal.examples.get_path('10740.shx'),'rb').read() == open(fname[:-1]+'x','rb').read()
    True
    >>> import os
    >>> os.remove(fname); os.remove(fname.replace('.shp','.shx'))

    """
    FORMATS = ['shp', 'shx']
    MODES = ['w', 'r', 'wb', 'rb']

    def __init__(self, *args, **kwargs):
        pysal.core.FileIO.FileIO.__init__(self, *args, **kwargs)
        self.dataObj = None
        if self.mode == 'r' or self.mode == 'rb':
            self.__open()
        elif self.mode == 'w' or self.mode == 'wb':
            self.__create()

    def __len__(self):
        if self.dataObj != None:
            return len(self.dataObj)
        else:
            return 0

    def __open(self):
        self.dataObj = shp_file(self.dataPath)
        self.header = self.dataObj.header
        self.bbox = self.dataObj.bbox
        try:
            self.type = STRING_TO_TYPE[self.dataObj.type()]
        except KeyError:
            raise TypeError('%s does not support shapes of type: %s'
                            % (self.__class__.__name__, self.dataObj.type()))

    def __create(self):
        self.write = self.__firstWrite

    def __firstWrite(self, shape):
        self.type = TYPE_TO_STRING[type(shape)]
        if self.type == 'POINT':
            if len(shape) == 3:
                self.type = 'POINTM'
            if len(shape) == 4:
                self.type = 'POINTZ'
        self.dataObj = shp_file(self.dataPath, 'w', self.type)
        self.write = self.__writer
        self.write(shape)

    def __writer(self, shape):
        if TYPE_TO_STRING[type(shape)] != self.type:
            raise TypeError("This file only supports %s type shapes" %
                            self.type)
        rec = {}
        rec['Shape Type'] = shp_file.SHAPE_TYPES[self.type]
        if self.type == 'POINT':
            rec['X'] = shape[0]
            rec['Y'] = shape[1]
            if len(shape) > 2:
                rec['M'] = shape[2]
            if len(shape) > 3:
                rec['Z'] = shape[3]
            shape = rec
        else:
            rec['BBOX Xmin'] = shape.bounding_box.left
            rec['BBOX Ymin'] = shape.bounding_box.lower
            rec['BBOX Xmax'] = shape.bounding_box.right
            rec['BBOX Ymax'] = shape.bounding_box.upper
            if self.type == 'POLYGON':
                holes = [hole[::-1] for hole in shape.holes if hole]
                    #holes should be in CCW order
                rec['NumParts'] = len(shape.parts) + len(holes)
                all_parts = shape.parts + holes
            else:
                rec['NumParts'] = len(shape.parts)
                all_parts = shape.parts
            partsIndex = [0]
            for l in [len(part) for part in all_parts][:-1]:
                partsIndex.append(partsIndex[-1] + l)
            rec['Parts Index'] = partsIndex
            verts = sum(all_parts, [])
            verts = [(x, y) for x, y in verts]
            rec['NumPoints'] = len(verts)
            rec['Vertices'] = verts
        self.dataObj.add_shape(rec)
        self.pos += 1

    def _read(self):
        try:
            rec = self.dataObj.get_shape(self.pos)
        except IndexError:
            return None
        self.pos += 1
        if self.dataObj.type() == 'POINT':
            shp = self.type((rec['X'], rec['Y']))
        elif self.dataObj.type() == 'POINTZ':
            shp = self.type((rec['X'], rec['Y']))
            shp.Z = rec['Z']
            shp.M = rec['M']
        else:
            if rec['NumParts'] > 1:
                partsIndex = list(rec['Parts Index'])
                partsIndex.append(None)
                parts = [rec['Vertices'][partsIndex[i]:partsIndex[
                    i + 1]] for i in xrange(rec['NumParts'])]
                if self.dataObj.type() == 'POLYGON':
                    is_cw = [pysal.cg.is_clockwise(part) for part in parts]
                    vertices = [part for part, cw in zip(parts, is_cw) if cw]
                    holes = [part for part, cw in zip(parts, is_cw) if not cw]
                    if not holes:
                        holes = None
                    shp = self.type(vertices, holes)
                else:
                    vertices = parts
                    shp = self.type(vertices)
            elif rec['NumParts'] == 1:
                vertices = rec['Vertices']
                if self.dataObj.type() == 'POLYGON' and not pysal.cg.is_clockwise(vertices):
                    ### SHAPEFILE WARNING: Polygon %d topology has been fixed. (ccw -> cw)
                    warn("SHAPEFILE WARNING: Polygon %d topology has been fixed. (ccw -> cw)" % (self.pos), RuntimeWarning)
                    print "SHAPEFILE WARNING: Polygon %d topology has been fixed. (ccw -> cw)" % (self.pos)

                shp = self.type(vertices)
            else:
                warn("Polygon %d has zero parts" % self.pos, RuntimeWarning)
                shp = self.type([[]])
                #raise ValueError, "Polygon %d has zero parts"%self.pos
        if self.ids:
            shp.id = self.rIds[self.pos - 1]  # shp IDs start at 1.
        else:
            shp.id = self.pos  # shp IDs start at 1.
        return shp

    def close(self):
        self.dataObj.close()
        pysal.core.FileIO.FileIO.close(self)




"""
Max p regionalization

Heuristically form the maximum number (p) of regions given a set of n
areas and a floor constraint.
"""

__author__ = "Serge Rey <srey@asu.edu>, David Folch <dfolch@fsu.edu>"


import pysal
from components import check_contiguity
import copy
import numpy as np
from pysal.region import randomregion as RR

__all__ = ["Maxp", "Maxp_LISA"]

LARGE = 10 ** 6
MAX_ATTEMPTS = 100


class Maxp:
    """Try to find the maximum number of regions for a set of areas such that
    each region combines contiguous areas that satisfy a given threshold
    constraint.


    Parameters
    ----------

    w               : W
                      spatial weights object

    z               : array
                      n*m array of observations on m attributes across n
                      areas. This is used to calculate intra-regional
                      homogeneity
    floor           : int
                      a minimum bound for a variable that has to be
                      obtained in each region
    floor_variable  : array
                      n*1 vector of observations on variable for the floor
    initial         : int
                      number of initial solutions to generate
    verbose         : binary
                      if true debugging information is printed
    seeds           : list
                      ids of observations to form initial seeds. If
                      len(ids) is less than the number of observations, the
                      complementary ids are added to the end of seeds. Thus
                      the specified seeds get priority in the solution

    Attributes
    ----------

    area2region     : dict
                      mapping of areas to region. key is area id, value is
                      region id
    regions         : list
                      list of lists of regions (each list has the ids of areas
                      in that region)
    p               : int
                      number of regions
    swap_iterations : int
                      number of swap iterations
    total_moves     : int
                      number of moves into internal regions

    Examples
    --------

    Setup imports and set seeds for random number generators to insure the
    results are identical for each run.

    >>> import numpy as np
    >>> import pysal
    >>> np.random.seed(100)

    Setup a spatial weights matrix describing the connectivity of a square
    community with 100 areas.  Generate two random data attributes for each
    area in the community (a 100x2 array) called z. p is the data vector used to
    compute the floor for a region, and floor is the floor value; in this case
    p is simply a vector of ones and the floor is set to three. This means
    that each region will contain at least three areas.  In other cases the
    floor may be computed based on a minimum population count for example.

    >>> import numpy as np
    >>> import pysal
    >>> np.random.seed(100)
    >>> w = pysal.lat2W(10,10)
    >>> z = np.random.random_sample((w.n,2))
    >>> p = np.ones((w.n,1), float)
    >>> floor = 3
    >>> solution = pysal.region.Maxp(w, z, floor, floor_variable=p, initial=100)
    >>> solution.p
    29
    >>> min([len(region) for region in solution.regions])
    3
    >>> solution.regions[0]
    [76, 66, 56]
    >>>

    """
    def __init__(self, w, z, floor, floor_variable,
                 verbose=False, initial=100, seeds=[]):

        self.w = w
        self.z = z
        self.floor = floor
        self.floor_variable = floor_variable
        self.verbose = verbose
        self.seeds = seeds
        self.initial_solution()
        if not self.p:
            self.feasible = False
        else:
            self.feasible = True
            best_val = self.objective_function()
            self.current_regions = copy.copy(self.regions)
            self.current_area2region = copy.copy(self.area2region)
            self.initial_wss = []
            self.attempts = 0
            for i in range(initial):
                self.initial_solution()
                if self.p:
                    val = self.objective_function()
                    self.initial_wss.append(val)
                    if self.verbose:
                        print 'initial solution: ', i, val, best_val
                    if val < best_val:
                        self.current_regions = copy.copy(self.regions)
                        self.current_area2region = copy.copy(self.area2region)
                        best_val = val
                    self.attempts += 1
            self.regions = copy.copy(self.current_regions)
            self.p = len(self.regions)
            self.area2region = self.current_area2region
            if verbose:
                print "smallest region ifs: ", min([len(region) for region in self.regions])
                raw_input='wait'

            self.swap()

    def initial_solution(self):
        self.p = 0
        solving = True
        attempts = 0
        while solving and attempts <= MAX_ATTEMPTS:
            regions = []
            enclaves = []
            if not self.seeds:
                candidates = copy.copy(self.w.id_order)
                candidates = np.random.permutation(candidates)
                candidates = candidates.tolist()
            else:
                seeds = copy.copy(self.seeds)
                nonseeds = [i for i in self.w.id_order if i not in seeds]
                candidates = seeds
                candidates.extend(nonseeds)
            while candidates:
                seed = candidates.pop(0)
                # try to grow it till threshold constraint is satisfied
                region = [seed]
                building_region = True
                while building_region:
                    # check if floor is satisfied
                    if self.check_floor(region):
                        regions.append(region)
                        building_region = False
                    else:
                        potential = []
                        for area in region:
                            neighbors = self.w.neighbors[area]
                            neighbors = [neigh for neigh in neighbors if neigh in candidates]
                            neighbors = [neigh for neigh in neighbors if neigh not in region]
                            neighbors = [neigh for neigh in neighbors if neigh not in potential]
                            potential.extend(neighbors)
                        if potential:
                            # add a random neighbor
                            neigID = np.random.randint(0, len(potential))
                            neigAdd = potential.pop(neigID)
                            region.append(neigAdd)
                            # remove it from candidates
                            candidates.remove(neigAdd)
                        else:
                            #print 'enclave'
                            #print region
                            enclaves.extend(region)
                            building_region = False
            # check to see if any regions were made before going to enclave stage
            if regions:
                feasible = True
            else:
                attempts += 1
                break
            self.enclaves = enclaves[:]
            a2r = {}
            for r, region in enumerate(regions):
                for area in region:
                    a2r[area] = r
            encCount = len(enclaves)
            encAttempts = 0
            while enclaves and encAttempts != encCount:
                enclave = enclaves.pop(0)
                neighbors = self.w.neighbors[enclave]
                neighbors = [neighbor for neighbor in neighbors if neighbor not in enclaves]
                candidates = []
                for neighbor in neighbors:
                    region = a2r[neighbor]
                    if region not in candidates:
                        candidates.append(region)
                if candidates:
                    # add enclave to random region
                    regID = np.random.randint(0, len(candidates))
                    rid = candidates[regID]
                    regions[rid].append(enclave)
                    a2r[enclave] = rid
                    # structure to loop over enclaves until no more joining is possible
                    encCount = len(enclaves)
                    encAttempts = 0
                    feasible = True
                else:
                    # put back on que, no contiguous regions yet
                    enclaves.append(enclave)
                    encAttempts += 1
                    feasible = False
            if feasible:
                solving = False
                self.regions = regions
                self.area2region = a2r
                self.p = len(regions)
            else:
                if attempts == MAX_ATTEMPTS:
                    print 'No initial solution found'
                    self.p = 0
                attempts += 1

    def swap(self):
        swapping = True
        swap_iteration = 0
        if self.verbose:
            print 'Initial solution, objective function: ', self.objective_function()
        total_moves = 0
        self.k = len(self.regions)
        changed_regions = [1] * self.k
        nr = range(self.k)
        while swapping:
            moves_made = 0
            regionIds = [r for r in nr if changed_regions[r]]
            np.random.permutation(regionIds)
            changed_regions = [0] * self.k
            swap_iteration += 1
            for seed in regionIds:
                local_swapping = True
                local_attempts = 0
                while local_swapping:
                    local_moves = 0
                    # get neighbors
                    members = self.regions[seed]
                    neighbors = []
                    for member in members:
                        candidates = self.w.neighbors[member]
                        candidates = [candidate for candidate in candidates if candidate not in members]
                        candidates = [candidate for candidate in candidates if candidate not in neighbors]
                        neighbors.extend(candidates)
                    candidates = []
                    for neighbor in neighbors:
                        block = copy.copy(self.regions[self.area2region[
                            neighbor]])
                        if check_contiguity(self.w, block, neighbor):
                            block.remove(neighbor)
                            fv = self.check_floor(block)
                            if fv:
                                candidates.append(neighbor)
                    # find the best local move
                    if not candidates:
                        local_swapping = False
                    else:
                        nc = len(candidates)
                        moves = np.zeros([nc, 1], float)
                        best = None
                        cv = 0.0
                        for area in candidates:
                            current_internal = self.regions[seed]
                            current_outter = self.regions[self.area2region[
                                area]]
                            current = self.objective_function([current_internal, current_outter])
                            new_internal = copy.copy(current_internal)
                            new_outter = copy.copy(current_outter)
                            new_internal.append(area)
                            new_outter.remove(area)
                            new = self.objective_function([new_internal,
                                                           new_outter])
                            change = new - current
                            if change < cv:
                                best = area
                                cv = change
                        if best:
                            # make the move
                            area = best
                            old_region = self.area2region[area]
                            self.regions[old_region].remove(area)
                            self.area2region[area] = seed
                            self.regions[seed].append(area)
                            moves_made += 1
                            changed_regions[seed] = 1
                            changed_regions[old_region] = 1
                        else:
                            # no move improves the solution
                            local_swapping = False
                    local_attempts += 1
                    if self.verbose:
                        print 'swap_iteration: ', swap_iteration, 'moves_made: ', moves_made
                        print 'number of regions: ', len(self.regions)
                        print 'number of changed regions: ', sum(
                            changed_regions)
                        print 'internal region: ', seed, 'local_attempts: ', local_attempts
                        print 'objective function: ', self.objective_function()
                        print 'smallest region size: ',min([len(region) for region in self.regions])
            total_moves += moves_made
            if moves_made == 0:
                swapping = False
                self.swap_iterations = swap_iteration
                self.total_moves = total_moves
            if self.verbose:
                print 'moves_made: ', moves_made
                print 'objective function: ', self.objective_function()

    def check_floor(self, region):
        selectionIDs = [self.w.id_order.index(i) for i in region]
        cv = sum(self.floor_variable[selectionIDs])
        if cv >= self.floor:
            #print len(selectionIDs)
            return True
        else:
            return False

    def objective_function(self, solution=None):
        # solution is a list of lists of region ids [[1,7,2],[0,4,3],...] such
        # that the first region has areas 1,7,2 the second region 0,4,3 and so
        # on. solution does not have to be exhaustive
        if not solution:
            solution = self.regions
        wss = 0
        for region in solution:
            selectionIDs = [self.w.id_order.index(i) for i in region]
            m = self.z[selectionIDs, :]
            var = m.var(axis=0)
            wss += sum(np.transpose(var)) * len(region)
        return wss

    def inference(self, nperm=99):
        """Compare the within sum of squares for the solution against
        simulated solutions where areas are randomly assigned to regions that
        maintain the cardinality of the original solution.

        Parameters
        ----------

        nperm       : int
                      number of random permutations for calculation of
                      pseudo-p_values

        Attributes
        ----------

        pvalue      : float
                      pseudo p_value

        Examples
        --------

        Setup is the same as shown above except using a 5x5 community.

        >>> import numpy as np
        >>> import pysal
        >>> np.random.seed(100)
        >>> w=pysal.weights.lat2W(5,5)
        >>> z=np.random.random_sample((w.n,2))
        >>> p=np.ones((w.n,1),float)
        >>> floor=3
        >>> solution=pysal.region.Maxp(w,z,floor,floor_variable=p,initial=100)

        Set nperm to 9 meaning that 9 random regions are computed and used for
        the computation of a pseudo-p-value for the actual Max-p solution. In
        empirical work this would typically be set much higher, e.g. 999 or
        9999.

        >>> solution.inference(nperm=9)
        >>> solution.pvalue
        0.1

        """
        ids = self.w.id_order
        num_regions = len(self.regions)
        wsss = np.zeros(nperm + 1)
        self.wss = self.objective_function()
        cards = [len(i) for i in self.regions]
        sim_solutions = RR.Random_Regions(ids, num_regions,
                                          cardinality=cards, permutations=nperm)
        cv = 1
        c = 1
        for solution in sim_solutions.solutions_feas:
            wss = self.objective_function(solution.regions)
            wsss[c] = wss
            if wss <= self.wss:
                cv += 1
            c += 1
        self.pvalue = cv / (1. + len(sim_solutions.solutions_feas))
        self.wss_perm = wsss
        self.wss_perm[0] = self.wss

    def cinference(self, nperm=99, maxiter=1000):
        """Compare the within sum of squares for the solution against
        conditional simulated solutions where areas are randomly assigned to
        regions that maintain the cardinality of the original solution and
        respect contiguity relationships.

        Parameters
        ----------

        nperm       : int
                      number of random permutations for calculation of
                      pseudo-p_values

        maxiter     : int
                      maximum number of attempts to find each permutation

        Attributes
        ----------

        pvalue      : float
                      pseudo p_value

        feas_sols   : int
                      number of feasible solutions found

        Notes
        -----

        it is possible for the number of feasible solutions (feas_sols) to be
        less than the number of permutations requested (nperm); an exception
        is raised if this occurs.

        Examples
        --------

        Setup is the same as shown above except using a 5x5 community.

        >>> import numpy as np
        >>> import pysal
        >>> np.random.seed(100)
        >>> w=pysal.weights.lat2W(5,5)
        >>> z=np.random.random_sample((w.n,2))
        >>> p=np.ones((w.n,1),float)
        >>> floor=3
        >>> solution=pysal.region.Maxp(w,z,floor,floor_variable=p,initial=100)

        Set nperm to 9 meaning that 9 random regions are computed and used for
        the computation of a pseudo-p-value for the actual Max-p solution. In
        empirical work this would typically be set much higher, e.g. 999 or
        9999.

        >>> solution.cinference(nperm=9, maxiter=100)
        >>> solution.cpvalue
        0.1

        """
        ids = self.w.id_order
        num_regions = len(self.regions)
        wsss = np.zeros(nperm + 1)
        self.cwss = self.objective_function()
        cards = [len(i) for i in self.regions]
        sim_solutions = RR.Random_Regions(ids, num_regions,
                                          cardinality=cards, contiguity=self.w,
                                          maxiter=maxiter, permutations=nperm)
        self.cfeas_sols = len(sim_solutions.solutions_feas)
        if self.cfeas_sols < nperm:
            raise Exception('not enough feasible solutions found')
        cv = 1
        c = 1
        for solution in sim_solutions.solutions_feas:
            wss = self.objective_function(solution.regions)
            wsss[c] = wss
            if wss <= self.cwss:
                cv += 1
            c += 1
        self.cpvalue = cv / (1. + self.cfeas_sols)
        self.cwss_perm = wsss
        self.cwss_perm[0] = self.cwss


class Maxp_LISA(Maxp):
    """Max-p regionalization using LISA seeds

    Parameters
    ----------

    w              : W
                     spatial weights object
    z              : array
                     nxk array of n observations on k variables used to
                     measure similarity between areas within the regions.
    y              : array
                     nx1 array used to calculate the LISA statistics and
                     to set the intial seed order
    floor          : float
                     value that each region must obtain on floor_variable
    floor_variable : array
                     nx1 array of values for regional floor threshold
    initial        : int
                     number of initial feasible solutions to generate
                     prior to swapping

    Attributes
    ----------

    area2region     : dict
                      mapping of areas to region. key is area id, value is
                      region id
    regions         : list
                      list of lists of regions (each list has the ids of areas
                      in that region)
    swap_iterations : int
                      number of swap iterations
    total_moves     : int
                      number of moves into internal regions


    Notes
    -----

    We sort the observations based on the value of the LISAs. This
    ordering then gives the priority for seeds forming the p regions. The
    initial priority seeds are not guaranteed to be separated in the final
    solution.

    Examples
    --------

    Setup imports and set seeds for random number generators to insure the
    results are identical for each run.

    >>> import numpy as np
    >>> import pysal
    >>> np.random.seed(100)

    Setup a spatial weights matrix describing the connectivity of a square
    community with 100 areas.  Generate two random data attributes for each area
    in the community (a 100x2 array) called z. p is the data vector used to
    compute the floor for a region, and floor is the floor value; in this case
    p is simply a vector of ones and the floor is set to three. This means
    that each region will contain at least three areas.  In other cases the
    floor may be computed based on a minimum population count for example.

    >>> w=pysal.lat2W(10,10)
    >>> z=np.random.random_sample((w.n,2))
    >>> p=np.ones(w.n)
    >>> mpl=pysal.region.Maxp_LISA(w,z,p,floor=3,floor_variable=p)
    >>> mpl.p
    30
    >>> mpl.regions[0]
    [99, 98, 89]

    """
    def __init__(self, w, z, y, floor, floor_variable, initial=100):

        lis = pysal.Moran_Local(y, w)
        ids = np.argsort(lis.Is)
        ids = ids[range(w.n - 1, -1, -1)]
        ids = ids.tolist()
        mp = Maxp.__init__(
            self, w, z, floor=floor, floor_variable=floor_variable,
            initial=initial, seeds=ids)

